/**
 * \file mgui/tooltext.h
 * \brief MGUI::TOOL_TEXT class definition
 *
 * \if NODOC
 * $Id: tooltext.h_v 1.12 2013/02/18 16:39:46 bcolombe Exp $
 *
 * $Log: tooltext.h_v $
 * Revision 1.12  2013/02/18 16:39:46  bcolombe
 * Add fonthandle for SetTextStyle
 *
 * Revision 1.11  2011/12/09 20:23:50  mju
 * No context in v_draw.
 *
 * Revision 1.10  2010/02/02 21:44:56  bcolombe
 * Add force option to SetActiveLine
 *
 * Revision 1.9  2009/05/26 21:03:06  bcolombe
 * Added mode
 *
 * Revision 1.8  2008/05/21 17:37:54  mju
 * Add setToolStyle.
 *
 * Revision 1.7  2008/05/21 17:33:55  mju
 * Add style_ControlFormEditHeight.
 *
 * Revision 1.6  2008/03/12 17:55:47  scowan
 * Added methods to allow editing of existing text.
 *
 * Revision 1.5  2008/02/28 18:31:24  scowan
 * Added setting height and adding leader line methods.
 *
 * Revision 1.4  2008/02/01 15:02:02  mju
 * Change button handlers to return bool.
 *
 * Revision 1.3  2008/01/17 15:20:10  mju
 * Add getHeight.
 *
 * Revision 1.2  2008/01/14 21:52:03  mju
 * Now working in basic mode.
 *
 * Revision 1.1  2008/01/14 16:33:11  mju
 * Initial revision
 *
 * Revision 1.6  2007/12/17 16:13:04  mju
 * \endif
**/

#ifndef	INC_MGUI_TOOLTEXT_H
#define	INC_MGUI_TOOLTEXT_H

#ifndef	INC_MGUI_TOOLBASE_H
	#include <mgui/toolbase.h>
#endif

#ifndef	INC_MI32_POLYLINE_H
	#include <mi32/polyline.h>
#endif

#ifndef	INC_MI32_ELEMSTYL_H
	#include <mi32/elemstyl.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

namespace MGUI {


//! Text graphic tool.
class TOOL_TEXT : public TOOL_BASE {
	public:

		enum TOOLSTYLE {
			TOOLSTYLE_Default =						0x0000,
			TOOLSTYLE_NoEditBaseLine =				0x0001,	//!< Don't allow editing of baseline or leaders
			TOOLSTYLE_NoLeaderLines =				0x0002,	//!< Don't allow creation/editing of leader lines
			TOOLSTYLE_NoStyleControls =			0x0004,	//!< Don't include style controls
			TOOLSTYLE_NoStyleAdvanced =			0x0008,	//!< Don't include "advanced" style button
			TOOLSTYLE_ControlFormEditHeight =	0x0010,	//!< Include "height" editing in "control form"
			};

		enum BASECURVE {
			BASECURVE_Straight,
			BASECURVE_Exact,
			BASECURVE_Spline
			};

		enum EDITMODE {
			EDITMODE_Automatic,
			EDITMODE_AddEnd,							//!< Add vertices to end of active line (default)
			EDITMODE_AddStart,						//!< Add vertices to start of active line
			EDITMODE_InsertVertex,					//!< Insert vertex
			EDITMODE_DragVertex,						//!< Drag nearest vertex
			EDITMODE_DeleteVertex,					//!< Delete nearest vertex
			EDITMODE_Move								//!< Move active line
			};

		class LEADER {
			public:

				LEADER ();

				LEADER (
					const POLYLINE& Line,
					INT32 ElemNum = -1,
					ELEMTYPE ElemType = ELEMTYPE_Empty
					);

				~LEADER ();

				INT32 GetElemNum (
					) const { return (m_ElemNum); }

				ELEMTYPE GetElemType (
					) const { return (m_ElemType); }

				const POLYLINE& GetLine (
					) const { return (m_Line); }

				POLYLINE& GetLine (
					) { return (m_Line); }

				const DPOINT3D& GetTiePoint (
					) const { return (m_TiePoint); }

				void SetElement (
					INT32 ElemNum,
					ELEMTYPE ElemType = ELEMTYPE_Empty
					) { m_ElemNum = ElemNum; m_ElemType = ElemType; }

				void SetLine (
					const POLYLINE& Line
					) { m_Line = Line; }

				void SetStartPoint (
					const DPOINT3D& point
					) { m_Line.SetVertex(0,point); }

				void SetTiePoint (
					const DPOINT2D& TiePoint
					) { m_TiePoint = TiePoint; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				POLYLINE m_Line;
				DPOINT3D m_TiePoint;
				INT32 m_ElemNum;
				ELEMTYPE m_ElemType;
				#endif
			};
		typedef MILIST<LEADER> LEADERLIST;

		//! Constructor.
		explicit TOOL_TEXT (
			TOOLCANVAS *canvas							//!< View to contain tool
			);

		//! Destructor.
		virtual ~TOOL_TEXT ();

		//! Clear string without clearing baseline or leaders.
		void ClearString (
			);

		//! Clear leader list without clearing baseline or string.
		void ClearLeaderList (
			);

		//! Clear source line without clearing baseline, string or leaders.
		//! The source line is the line to follow if text is to be moved "along" the line
		void ClearSourceLine (
			);

		//! Get base curvature.
		BASECURVE GetBaseCurve (
			) const { return (m_BaseCurve); }

		//! Get base line.
		const POLYLINE& GetBaseLine (
			) const { return (m_BaseLine); }

		//! Get line editing mode.
		EDITMODE GetEditMode (
			) const { return (m_EditMode); }

		//! Get height in tool units.
		double GetHeight (
			) const { return (m_Height); }

		//! Get list of leader lines.
		const LEADERLIST& GetLeaderList (
			) const { return (m_LeaderList); }

		//! Get text string.
		const MISTRING& GetText (
			) const { return (m_Text); }

		//! Get text style.
		const TEXTSTYLE& GetTextStyle (
			) const { return (m_TextStyle); }

		//! Get tool style.
		TOOLSTYLE GetToolStyle (
			) const { return (m_ToolStyle); }

		//! Add new leader line and make it "active".
		void LeaderLineAdd (
			);

		//! Add new leader line and make it "active".
		void LeaderLineAdd (
			const LEADER& Leader
			);

		//! Remove active leader line.
		void LeaderLineRemoveActive (
			);

		//! Reverse order of baseline points.
		void ReverseBaseLine (
			);

		//! Set active "line" to next.
		void SetActiveNext (
			);

		//! Set active "line" to previous.
		void SetActivePrevious (
			);

		//! Set base curvature.
		void SetBaseCurve (
			BASECURVE BaseCurve
			);

		//! Set base line.
		void SetBaseLine (
			const POLYLINE& BaseLine,
			BASECURVE BaseCurve
			);

		//! Set caret position
		void SetCaretPos (
			INT32 row,
			INT32 column
			);

		//! Set line editing mode.
		void SetEditMode (
			EDITMODE EditMode							//!< New editing mode
			) { m_EditMode = EditMode; }

		//! Set height in tool units.
		void SetHeight (
			double height
			);

		//! Set list of leader lines.
		void SetLeaderList (
			const LEADERLIST& LeaderList
			);

		//! Set map scale for height computation.
		void SetMapScale (
			double MapScale
			);

		//! Set source line
		//! The source line is the line to follow if text is to be moved "along" the line
		void SetSourceLine (
			const POLYLINE& SourceLine
			);

		//! Set text string.
		void SetText (
			const MISTRING& text
			);

		//! Set text style.
		void SetTextStyle (
			const TEXTSTYLE& TextStyle,
			int fonthandle = -1
			);

		//! Set tool style.
		void SetToolStyle (
			TOOLSTYLE ToolStyle
			) { m_ToolStyle = ToolStyle; }

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT

		class CONTROLFORM;

		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_SetBasePoint,
			ZONE_MoveBaseLine,
			ZONE_Vertex,
			ZONE_Height,
			ZONE_RotateBaseLine,
			ZONE_InsertVertex,
			};

		// Tool settings
		TOOLSTYLE m_ToolStyle;
		// Tool values.
		MISTRING m_Text;						  	// Text string being edited
		TEXTSTYLE m_TextStyle;				  	// Style information for text
		double m_Height;						  	// Height of text in tool units
		double m_CanvasScale;				  	// Map scale of canvas
		BASECURVE m_BaseCurve;					// Baseline curve
		POLYLINE m_BaseLine;						// Baseline
		POLYLINE m_SourceLine;					// Source line to follow if text is to be moved "along" the line
		POLYLINE *m_pCurLine;					// Current line (base or leader) being edited
		LEADERLIST m_LeaderList;				// List of leader lines
		// Values used during interaction
		ZONE m_PointerZone;						// Current cursor zone
		EDITMODE m_EditMode;
		bool m_FlipHeight;
		INT32 m_CaretRow;
		INT32 m_CaretCol;
		INT32 m_CurVertex;						// Closest vertex if required
		DPOINT2D m_prevpt;
		double m_LineAngle;
		CONTROLFORM *m_pControlForm;
		
		// Methods
		void AdjustBaseLine ();
		void AdjustHeight (const DPOINT2D& pt);
		void AdjustTool (const DPOINT2D& pt, KEYSTATE keystate, bool firsttime=false);
		void ApplyTrans (const TRANS2D_AFFINE& trans);
		void ComputeCaretCoords (DPOINT2D& BottomPt, DPOINT2D& TopPt);
		void ComputeLineAngle ();
		void ComputePointerZone (const DPOINT2D& pt);
		void DeleteVertices (const DPOINT2D& pt);
		void DrawBaseLine (MGD::CONTEXT *dc);
		void DrawLeaderLines (MGD::CONTEXT *dc);
		void DrawLine (const POLYLINE& line, MGD::CONTEXT *dc);
		void DrawMarkers (MGD::CONTEXT *dc);
		void DrawText (MGD::CONTEXT *dc);
		void GetActualBaseLine (POLYLINE& baseline);
		void GetCurTextStyle (TEXTSTYLE& TextStyle);
		void SetActiveLine (POLYLINE *pActiveLine, bool bForce = false);
		void SetContextStyle (MGD::CONTEXT *dc);
		double TransHeightDtoT (double height) const;
		double TransHeightTtoD (double height) const;
		void UpdateCursor () const;

		// TOOL_BASE overrides.
		virtual MGUI::FORM* v_CreateControlForm (LAYOUT_PANE_BASE& ParentPane, CTRLOPT ctrlopt);
		virtual void v_DoDraw ();
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual void v_OnClear ();
		virtual bool v_OnKey (MGUI::POINT point, KEYCODE keycode, KEYSTATE keystate);
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual bool v_OnPointerButtonUp (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnSetTransToWindow ();
		
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI


#endif	// INC_MGUI_TOOLTEXT_H
