/** 
 * \file mgui/wysiwyg.h
 * \brief Definitions for MGUI::CTRL_EDIT_TEXT_WYSIWYG classes.
 *
 * WYSIWYG = What You See Is What You Get
 *
 * \if NODOC
 * $Id: wysiwyg.h_v 1.19 2013/01/18 22:30:37 mju Exp $
 *
 * $Log: wysiwyg.h_v $
 * Revision 1.19  2013/01/18 22:30:37  mju
 * Remove pointless comment.
 *
 * Revision 1.18  2011/11/09 15:46:27  mju
 * Remove mfc defns.
 *
 * Revision 1.17  2011/09/01 18:06:30  mju
 * Exclude macro from dox.
 *
 * Revision 1.16  2007/03/26 14:40:18  dwilliss
 * added toggle for when ywsiwyg mode gets toggled
 *
 * Revision 1.15  2007/03/09 18:12:33  dwilliss
 * Added method to turn wysiwyg on and off
 *
 * Revision 1.1  2002/10/09 22:26:51  dwilliss
 * Initial revision
 *
 * \endif
 **/

#ifndef	INC_MGUI_WYSIWYG_H
#define	INC_MGUI_WYSIWYG_H

#ifndef INC_MGUI_CTRL_H
	#include <mgui/ctrl.h>
#endif

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
struct TEXTSTYLE;
#endif


namespace MGUI {

//===================================================================================================================
//! Multi-line text 'edit' control.
class CTRL_EDIT_TEXT_WYSIWYG : public MGUI::CTRL {
	public:

		enum FLAGS {
			FLAG_Default =			0x0000,
			FLAG_ReadOnly =		0x0001,		//!< User cannot change value
			};

		//! Constructor.
		CTRL_EDIT_TEXT_WYSIWYG (
			);

		//! Destructor.
		virtual ~CTRL_EDIT_TEXT_WYSIWYG (
			);

		//! Determine if the last edit operation can be undone by a call to the Undo member function.
		bool CanUndo (
			) const;
			;

		//! Clear entire control to empty string.
		void ClearValue (
			bool notify = true						//!< Call OnChangeValue() if control has already been created
			);

		//! Clear (delete) the current selection
		void ClearSelection (
			);

		//! Cut the current selection to the clipboard
		void CutSelection (
			);

		//! Copy the current selection to the clipboard
		void CopySelection (
			);

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			int width = 0,								//!< Width in 'typical' characters, minimum if can expand based on sizealign
			int height = 0,							//!< Height lines, minimum if can expand based on sizealign
			FLAGS flags = FLAG_Default,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get the cursor position in characters from the start of the text
		INT32 GetCursorPos (
			) const;

		//! Determines the topmost visible line in an edit control.
		//! Returns the zero-based index of the topmost visible line. For single-line edit controls, 
		//!	the return value is 0.
		int GetFirstVisibleLine (
			) const;

		//! Get the number of lines in the edit control.
		int GetLineCount (
			) const;

		//! Retrieves the line number of the line that contains the specified character index.
		//! Returns the zero-based line number of the line containing the character index specified by nIndex. 
		//! If nIndex is -1, the number of the line that contains the first character of the selection 
		//! is returned. If there is no selection, the current line number is returned.
		int GetLineFromChar (
			int nIndex = -1
			) const;

		//! Retrieves the character index of a line within a multiple-line edit control.
		//! Returns the character index of the line specified in nLine or -1 if the specified line 
		//! number is greater then the number of lines in the edit control.
		int GetLineIndex (
			int nLine = -1
			);

		//! Retrieves the character index of a line within a multiple-line edit control.
		//! Return value is the length (in bytes) of the line specified by nLine.
		//! nLine specifies the character index of a character in the line whose length is to be 
		//! retrieved. If this parameter is -1, the length of the current line (the line that 
		//! contains the caret) is returned, not including the length of any selected text 
		//! within the line. 
		int GetLineLength (
			int nLine = -1
			);

		//! Returns true if contents of the edit-control have been modified, false if not.
		//!	Use SetModify() to reset.
		bool GetModify (
			) const;
			;

		//! Get the current selection of the edit control.
		//!	If nStartChar == nEndChar, nothing is selected.
		void GetSelection (
			int& nStartChar,		//!< Receives the first character in the current selection
			int& nEndChar			//!< Receives the first non-selected character past the end of the current selection
			) const;

		//! Get the text style at the specified offset
		void GetStyle (
			TEXTSTYLE& style,
			int offset = -1		//!< Offset to get style at.  -1 for style at cursor
			) const;

		//! Get current string value.
		const MISTRING& GetValue (
			) const { return (m_value); }

		//! Determine if WYSIWYG editing is on or off.
		bool GetWYSIWYG (
			) const;

		//! Hide the cursor (a.k.a. caret).
		void HideCursor (
			);

		//! Scrolls the text of a multiple-line edit control.
		void LineScroll (
			int nLines,			//!< Specifies the number of lines to scroll vertically
			int nChars = 0		//!< Specifies the number of character positions to scroll horizontally
			);

		//! Paste the clipboard contents at the current location
		void Paste ();

		//! Get the current cursor (a.k.a. caret) location.
		void SetCursorPos (
			int nCharIndex,
			bool bScroll = true	//!< True to scroll so cursor is visible in the window.
			);

		//! Get the current cursor (a.k.a. caret) location
		void SetCursorPos (
			int line,
			int col,
			bool bScroll = true	//!< True to scroll so cursor is visible in the window.
			);

		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS Delegate
			) { m_DelegateOnChangeValue = Delegate; }

		//! Set the delegate to call when the cursor moves.
		//! The delegate will get the new cursor position and should return true
		//! to allow the motion, false to disallow it.
		void SetDelegateOnCursorMotion (
			DELEGATE_BOOL_INT32 Delegate
			) { m_DelegateOnCursorMotion = Delegate; }

		void SetDelegateOnFocus (
			DELEGATE_VOID_NOPARMS Delegate
			) { m_DelegateOnFocus = Delegate; }

		void SetDelegateOnLosingFocus (
			DELEGATE_VOID_NOPARMS Delegate
			) { m_DelegateOnLosingFocus = Delegate; }

		void SetDelegateOnToggleWYSIWYG (
			DELEGATE_VOID_BOOL Delegate
			) { m_DelegateOnToggleWYSIWYG = Delegate; }

		void SetDelegateOnUserEdit (
			DELEGATE_VOID_NOPARMS Delegate
			) { m_DelegateOnUserEdit = Delegate; }

		//! Set or clear the "modified" state of the edit control.
		void SetModify (
			bool modified = true
			);

		//! Set whether value is read-only or not.
		void SetReadOnly (
			bool ReadOnly = true						//!< True if read only, false if not
			);

		//! Select a range of text
		void SetSelection (
			int start,
			int end
			);

		//! Get the text style of the current selection, or at the current
		//! cursor position (if nothing is selected)
		void SetStyle (
			const TEXTSTYLE& style
			);

		//! Get the text style of a range of text.
		void SetStyle (
			const TEXTSTYLE& style,
			int start,				//!< Starting offset to modify
			int end					//!< Ending offset to modify
			);

		//! Set current string value with validation.
		//! OnValidate() will be called and if valid, the string will be updated.
		//! No comparison is made with the current string value.
		void SetValue (
			const MIUNICODE* string,					//!< New string to set
			bool notify = true						//!< Call OnChangeValue() if string actually updated
			);

		void SetWordWrap (
			bool bWordWrap = true
			);

		//! Turn WYSIWYG editing on or off.
		void SetWYSIWYG (
			bool bvalue
			);

		//! Show the cursor (a.k.a. caret)
		void ShowCursor (
			);

		//! Undo the last edit
		void Undo ();

	protected:

		virtual void v_OnActivate ();

		//! Called when value is changed after validation.
		//! Derived class must call corresponding base class method BEFORE performing its own processing.
		//! If the user modifies the text in the edit control this will not be called until the control loses "focus".
		virtual void v_OnChangeValue ();

		//! Called when user modifies the text in the edit control.
		//! Derived class must call corresponding base class method BEFORE performing its own processing.
		//! The actual value does not get updated until the control loses focus or GetValue() is called.
		virtual void v_OnUserEdit ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		bool m_UserEdited;
		Widget m_TraversalWidget;
		int m_TraversalDir;
		bool m_InternalModify;
		bool m_HasValue;
		MISTRING m_value;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeValue;
		DELEGATE_BOOL_INT32 m_DelegateOnCursorMotion;
		DELEGATE_VOID_NOPARMS m_DelegateOnFocus;
		DELEGATE_VOID_NOPARMS m_DelegateOnLosingFocus;
		DELEGATE_VOID_NOPARMS m_DelegateOnUserEdit;
		DELEGATE_VOID_BOOL m_DelegateOnToggleWYSIWYG;

		virtual void v_SetFocus() const;

		bool GetEditStr (MISTRING& str) const;
		void UpdateCtrl (const MISTRING& string);

		//! Called internally to update control-specific data from edit control.
		virtual void UpdateValue (bool notify = true);

		static void CB_Activate (Widget, CTRL_EDIT_TEXT_WYSIWYG*, void*);
		static void CB_Focus (Widget, CTRL_EDIT_TEXT_WYSIWYG*, void*);
		static void CB_LosingFocus (Widget, CTRL_EDIT_TEXT_WYSIWYG*, void*);
		static void CB_ModifyVerify (Widget, CTRL_EDIT_TEXT_WYSIWYG*, void*);
		static void CB_ValueChanged (Widget, CTRL_EDIT_TEXT_WYSIWYG*, void*);
		static void CB_MotionVerify (Widget, CTRL_EDIT_TEXT_WYSIWYG*, void*);
		static void CB_ToggleWYSIWYG (Widget, CTRL_EDIT_TEXT_WYSIWYG*, void*);

		// Unimplemented
		CTRL_EDIT_TEXT_WYSIWYG (const CTRL_EDIT_TEXT_WYSIWYG&);
		CTRL_EDIT_TEXT_WYSIWYG& operator= (const CTRL_EDIT_TEXT_WYSIWYG&);

		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::CTRL_EDIT_TEXT_WYSIWYG::FLAGS)
#endif


//===================================================================================================================

}	// End of MGUI namespace

#endif	// INC_MGUI_CTRL_H
