/**
 * \file memalloc.h mi32/memalloc.h
 * \brief Memory allocation functions
 *
 * \if NODOC
 * $Id: memalloc.h_v 1.2 2008/08/28 14:03:27 mju Exp $
 *
 * $Log: memalloc.h_v $
 * Revision 1.2  2008/08/28 14:03:27  mju
 * Fix doxygen code block.
 *
 * Revision 1.1  2005/07/14 17:28:22  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MEMALLOC_H
#define	INC_MI32_MEMALLOC_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_STDLIB_H
#include <stdlib.h>			// for free()
#define	INC_STDLIB_H
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif


//!\addtogroup Mm Memory Allocation Functions (Mm)
//!@{

//! Free a pointer and set it's value to NULL.
#if defined(__cplusplus)
template <class _CT> inline void MmFree(_CT *& p) { if (p != 0) { free(p); p = NULL; } }
#elif !defined(GENERATING_DOXYGEN_OUTPUT)
#define	MmFree(p)		do {if ((p) != NULL) {free((p));(p) = NULL;}} while (0)
#endif

#if defined(__cplusplus)
extern "C" {
#endif

//! Allocate memory - Return error on failure.
//!
//! Returns error code if allocation fails.
//! Example:
//! @code
//!    UINT8* buf;
//!    err = MmAlloc((void**)&buf, size);
//!    if (err < 0) return(SetErrPosn(err));
//! @endcode
LIBEXPORT ERRVALUE MmAlloc (
	void** ptr,									//!< Pointer to memory allocated and returned
	int size										//!< Size to allocate in bytes
	);

//! Allocate and clear memory - Return error on failure.
//! @see MmAlloc()
LIBEXPORT ERRVALUE MmAllocC (
	void** ptr,									//!< Pointer to memory allocated and returned
	int size										//!< Size to allocate in bytes
	);

//! Allocate and clear memory - Throw exception of failure.
//! @see MmAlloc()
LIBEXPORT void MmAllocCExc (
	void** ptr,									//!< Pointer to memory allocated and returned
	int size										//!< Size to allocate in bytes
	);

//! Allocate memory - Throw exception of failure.
//! @see MmAlloc()
LIBEXPORT void MmAllocExc (
	void** ptr,									//!< Pointer to memory allocated and returned
	int size										//!< Size to allocate in bytes
	);

//! Resize memory buffer, retaining existing contents - Return error on failure.
//! @see MmAlloc()
LIBEXPORT ERRVALUE MmRealloc (
	void** ptr,									//!< Pointer to memory to reallocate and returned
	int newsize									//!< New size in bytes
	);

#if defined(__cplusplus)
//! Resize memory buffer and clear new part - Return error on failure.
//! @see MmAlloc()
LIBEXPORT ERRVALUE MmReallocC (
	void** ptr,									//!< Pointer to memory to reallocate and returned
	int newsize,								//!< New size in bytes
	int oldsize = 0							//!< Size of portion to not clear in bytes
	);
#elif !defined(GENERATING_DOXYGEN_OUTPUT)	//  normal "C" 
LIBEXPORT ERRVALUE MmReallocC (
	void** ptr,									//!< Pointer to memory to reallocate and returned
	int newsize,								//!< New size in bytes
	int oldsize									//!< Size of portion to not clear in bytes
	);
#endif

#if defined(__cplusplus)
//! Resize memory buffer and clear new part - Throw exception on failure.
//! @see MmAlloc()
LIBEXPORT void MmReallocCExc (
	void** ptr,									//!< Pointer to memory to reallocate and returned
	int newsize,								//!< New size in bytes
	int oldsize = 0							//!< Size of portion to not clear in bytes
	);
#endif

#if defined(__cplusplus)
//! Resize memory buffer, discarding existing contents - Return error on failure.
//! @see MmAlloc()
inline ERRVALUE MmReallocD (
	void** ptr,									//!< Pointer to memory to reallocate and returned
	int newsize									//!< New size in bytes
	) {
	if (*ptr != 0) {
		free(*ptr);
		}
	return (MmAlloc(ptr,newsize));
	}
#endif

#if defined(__cplusplus)
//! Resize memory buffer, discarding existing contents - Throw exception on failure.
//! @see MmAlloc()
inline void MmReallocDExc (
	void** ptr,									//!< Pointer to memory to reallocate and returned
	int newsize									//!< New size in bytes
	) {
	if (*ptr != 0) {
		free(*ptr);
		}
	MmAllocExc(ptr,newsize);
	return;
	}
#endif

#if defined(__cplusplus)
//! Resize memory buffer, retaining existing contents - Throw exception on failure.
//! @see MmAlloc()
LIBEXPORT void MmReallocExc (
	void** ptr,									//!< Pointer to memory to reallocate and returned
	int newsize									//!< New size in bytes
	);
#endif

#if defined(__cplusplus)
}
#endif

//!@}

#undef LIBEXPORT

#endif	// INC_MI32_MEMALLOC_H
