/**
 * \file mnetfunc.h <mi32/mnetfunc.h>
 * \brief Header file for network analysis library functions
 *
 * \if NODOC
 * $Id: mnetfunc.h_v 1.34 2007/09/24 15:28:13 scowan Exp $
 *
 * $Log: mnetfunc.h_v $
 * Revision 1.34  2007/09/24 15:28:13  scowan
 * Added obj item fwd decl.
 *
 * Revision 1.33  2007/04/27 16:25:46  mju
 * Use objectname hdr.
 *
 * Revision 1.32  2007/02/06 17:03:54  scowan
 * Changed init to take a objitem.
 *
 * Revision 1.31  2005/03/31 16:57:18  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.30  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.29  2002/08/23 14:52:44  scowan
 * Fixed deprecated stuff.
 *
 * Revision 1.28  2002/02/20 23:04:26  scowan
 * More const.
 *
 * Revision 1.27  2002/02/20 20:32:29  vdronov
 * nothing
 *
 * Revision 1.26  2002/02/20 16:47:29  scowan
 * More const.
 *
 * Revision 1.25  2002/02/20 16:36:28  scowan
 * More const.
 *
 * Revision 1.24  2001/01/17 22:45:37  vdronov
 * added multiroute functions
 *
 * Revision 1.23  2000/06/23 20:44:07  sparsons
 * Genitor documentation.
 *
 * Revision 1.22  2000/06/22 21:00:20  mju
 * Add stddefns so compiles in isolation.
 *
 * Revision 1.21  1999/04/08 21:05:12  mju
 * Add inclusion guarads.
 *
 * Revision 1.20  1999/03/09  00:50:13  vdronov
 * fixed mistyping
 *
 * Revision 1.19  1999/03/05  00:04:02  vdronov
 * added IsUpdated function
 *
 * Revision 1.18  1999/01/05  19:12:40  scowan
 * Swapped multiple keywords
 *
 * Revision 1.1  1998/08/28  22:59:35  vdronov
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MNETFUNC_H
#define	INC_MI32_MNETFUNC_H

#ifndef	INC_RVC_OBJECTNAME_H
#include <rvc/objectname.h>
#endif

namespace RVC {
	class OBJITEM;
	}

struct COLOR;

/***********************************************************************
*
*		Defines
*
************************************************************************/

#define		NETWORK_TABLE_Line						0
#define		NETWORK_TABLE_Turn						1
#define		NETWORK_TABLE_Barrier					2
#define		NETWORK_TABLE_Angle						3
#define		NETWORK_TABLE_Count						4

#define		NETWORK_ALLOCATION_In					0
#define		NETWORK_ALLOCATION_Out					1

#define		NETWORK_LINE_DIRECTION_FromTo			0
#define		NETWORK_LINE_DIRECTION_ToFrom			1

#define		NETWORK_LINE_DIRECTION_Disable		0
#define		NETWORK_LINE_DIRECTION_Enable			1

#define		NETWORK_NODE_BARRIER_Off				0
#define		NETWORK_NODE_BARRIER_On					1

#define		NETWORK_ANGLE_MAX							360


/***********************************************************************
*
*		Structures
*
************************************************************************/

typedef void *MNETWORKHANDLE;
typedef void *MNETWORKSTOPHANDLE;
typedef void *MNETWORKCENTERHANDLE;
typedef void *MNETWORKROUTEHANDLE;
typedef void *MNETWORKMULTIROUTEHANDLE;
typedef void *MNETWORKALLOCATIONHANDLE;

typedef int (*MnetworkLineNameCallbackFunc)(void*, INT32, MIUNICODE*, int);

/***********************************************************************
*
*		Function Prototypes
*
************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif



//!:Associate with "Network Analysis Library Functions"
//!\addtogroup NetworkAnalysis Network Analysis Functions
//!@{

// Main functions

//! Init all resources associated with network.
int MnetworkMainInit (
	MNETWORKHANDLE *handle, 				//!< Handle associated with network returned 
	const RVC::OBJITEM& VectObjItem		//!< OBJITEM of network vector object 
	);

//! Free all resources associated with network.
int MnetworkMainFree (
	MNETWORKHANDLE *handle					//!< Handle returned by MnetworkMainInit function 
	);

//! Create stop handle to maintain stops array.
int MnetworkMainCreateStop (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	MNETWORKSTOPHANDLE *stopHandle		//!< Stop handle to maintain stops array
	);

//! Create center handle to maintain centers array.
int MnetworkMainCreateCenter (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	MNETWORKCENTERHANDLE *centerHandle	//!< Center handle to maintain centers array
	);

//! Set all resources to default.
int MnetworkMainSetDefault (
	MNETWORKHANDLE handle					//!< Handle returned by MnetworkMainInit function  
	);

//! Is there "network related" table name.
//!
//! @return 1 if yes, 0 if no or error code < 0.
//!
//! Network related table types:			
//!			NETWORK_TABLE_Line						NETWORK_TABLE_Turn									
//!			NETWORK_TABLE_Barrier					NETWORK_TABLE_Angle						
int MnetworkMainIsTable (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	const RVC::OBJECTNAME& tablename, 					//!< Table name to check
	UINT32 tabletype							//!< Type of "network related" table
	);
	
//! Get databases table names array associated with this network by specific type.
//!
//! Network related table types:			
//!			NETWORK_TABLE_Line						NETWORK_TABLE_Turn									
//!			NETWORK_TABLE_Barrier					NETWORK_TABLE_Angle						
int MnetworkMainGetTableNames (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	RVC::OBJECTNAME **tablename, 				//!< Table name array allocated and returned
	INT32 *tablecount, 						//!< Size of table name array 
	UINT32 tabletype							//!< Type of "network related" table
	);
	
//! Read "network related" table.
//! 
//! Network related table types:
//!			NETWORK_TABLE_Line						NETWORK_TABLE_Turn						
//!			NETWORK_TABLE_Barrier					NETWORK_TABLE_Angle						 
int MnetworkMainReadTable (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	const RVC::OBJECTNAME& tablename, 					//!< Table name to read
	UINT32 tabletype							//!< Type of "network related" table
	);

//! Write "network related" table.
//!
//! Network related table types:
//!			NETWORK_TABLE_Line						NETWORK_TABLE_Turn						
//!			NETWORK_TABLE_Barrier					NETWORK_TABLE_Angle
int MnetworkMainWriteTable (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	RVC::OBJECTNAME& tablename, 					//!< Table name to write, RETURNED if table created
	UINT32 tabletype							//!< Type of "network related" table
	); 

//! Set line name as returned by callback.
int MnetworkMainSetLineNameAsCallback (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function
	MnetworkLineNameCallbackFunc cbfunc,//!< Function that will defines line name 
	void* cbdata								//!< Data that will be passed to callback as argument
	);

//! Set line name as database field.
int MnetworkMainSetLineNameAsTableAndField (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function 
	RVC::OBJECTNAME& tablename, 					//!< Table name 
	RVC::OBJECTNAME& fieldname						//!< Field name
	);

//! Get number of lines in network.
//!
//! @return Number of lines or error code < 0.
INT32 MnetworkLineGetNumber (
	MNETWORKHANDLE handle					//!< Handle returned by MnetworkMainInit function  
	);

//! Is attributes of lines updated.
//!
//! @return Handle returned by MnetworkMainInit function.
int MnetworkLineIsUpdated (
	MNETWORKHANDLE handle					//!< Handle returned by MnetworkMainInit function  
	);

//! Set impedance of specified line.
//!
//! Direction on line:			
//!		NETWORK_LINE_DIRECTION_FromTo		 
//!		NETWORK_LINE_DIRECTION_ToFrom		 
int MnetworkLineSetImpedance (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line, 								//!< Line
	int direction, 							//!< Direction on line
	DOUBLE impedance							//!< Impedance value to set 
	);

//! Get impedance of specified line.
//!
//!Direction on line:			
//!		NETWORK_LINE_DIRECTION_FromTo		 
//!		NETWORK_LINE_DIRECTION_ToFrom
int MnetworkLineGetImpedance (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line, 								//!< Line
	int direction, 							//!< Direction on line
	DOUBLE *impedance							//!< Impedance value of line returned
	);

//! Set demand of specified line.
int MnetworkLineSetDemand (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line, 								//!< Line
	DOUBLE demand								//!< Demand value to set
	);

//! Get demand of specified line.
int MnetworkLineGetDemand (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line, 								//!< Line
	DOUBLE *demand								//!< Demand value returned
	);

//! Set direction state of specified line.
//!
//! Direction on line:			
//!		NETWORK_LINE_DIRECTION_FromTo		 
//!		NETWORK_LINE_DIRECTION_ToFrom		   			
//!
//!	State of line:
//!		NETWORK_LINE_DIRECTION_Disable	 			
//!		NETWORK_LINE_DIRECTION_Enable		 
int MnetworkLineSetDirectionState (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line, 								//!< Line
	int direction, 							//!< Direction on line
	int state									//!< State of line to set
	);

//! Get direction state of specified line.
//!
//! Direction on line:			
//!		NETWORK_LINE_DIRECTION_FromTo		 
//!		NETWORK_LINE_DIRECTION_ToFrom		   			
//!		
//!	State of line:
//!		NETWORK_LINE_DIRECTION_Disable	 			
//!		NETWORK_LINE_DIRECTION_Enable		 
int MnetworkLineGetDirectionState (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line, 								//!< Line
	int direction, 							//!< Direction on line
	int *state									//!< State of line returned
	);

//! Get name of specified line.
int MnetworkLineGetName (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line, 								//!< Line
	MIUNICODE *name, 							//!< Unicode name of line returned (buffer should be allocated)
	int size										//!< Size of allocated buffer in unicode characters
	);

//! Get start node of specified line.
INT32 MnetworkLineGetNodeFrom (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line									//!< Line
	);

//! Get end node of specified line.
INT32 MnetworkLineGetNodeTo (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 line									//!< Line
	);

//! Get closest line to given point.
INT32 MnetworkLineGetClosest (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	DPOINT2D *point, 							//!< Point of interest in vector object coordinates
	DOUBLE radius								//!< Radius of interest in vector object coordinates
	);

//! Get number of nodes in network.
//!
//! @return Number of nodes or error code < 0.
INT32 MnetworkNodeGetNumber (
	MNETWORKHANDLE handle					//!< Handle returned by MnetworkMainInit function  
	);

//! Set barrier state of specified node.
//!
//! State of node:			
//!		NETWORK_NODE_BARRIER_Off			
//! 	NETWORK_NODE_BARRIER_On	
int MnetworkNodeSetBarrierState (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 node, 								//!< Node value
	int state									//!< State of node
	);

//! Get barrier state of specified node.
//!
//! State of node:			
//!		NETWORK_NODE_BARRIER_Off			
//!		NETWORK_NODE_BARRIER_On	
int MnetworkNodeGetBarrierState (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function 
	INT32 node, 								//!< Node value
	int *state									//!< State of node
	);

//! Get name of specified line (generated from line names).
int MnetworkNodeGetName (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 node, 								//!< Node value
	MIUNICODE *name, 							//!< Name string
	int size										//!< Size of name string
	);

//! Get number of incoming(outgoing) lines in lines array for specified node.
//!
//! @return Number of lines or error code < 0.
int MnetworkNodeGetNumberOfLines (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 node									//!< Node value
	);

//! Get line at specified position in incoming(outgoing) lines array for specified node.
//!
//! @return Line or error code < 0.
INT32 MnetworkNodeGetLine (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 node, 								//!< Node value
	int position								//!< Position of line
	);

//! Get closest node to given point.
//!
//! @return node or error code < 0.
INT32 MnetworkNodeGetClosest (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	DPOINT2D *point, 							//!< Point of interest in vector object coordinates
	DOUBLE radius								//!< Radius of interest in vector object coordinates
	);

//! Is barrier states updated.
//!
//! @return 1 if yes, 0 if no or error code < 0.
int MnetworkBarrierIsUpdated (
	MNETWORKHANDLE handle					//!< Handle returned by MnetworkMainInit function  
	);

//! Set impedance of specified turn.
int MnetworkTurnSetImpedance (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 node, 								//!< Node value
	int in, 										//!< Position of incoming line of specified node
	int out, 									//!< Position of outgoing line of specified node
	DOUBLE impedance							//!< Angle impedance of specified turn
	);

//! Get impedance of specified turn.
int MnetworkTurnGetImpedance (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 node, 								//!< Node value
	int in, 										//!< Position of incoming line of specified node
	int out, 									//!< Position of outgoing line of specified node
	DOUBLE *impedance							//!< Angle impedance of specified turn
	);

//! Angle impedance of specified turn.
int MnetworkTurnGetAngle (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	INT32 node, 								//!< Node value
	int in, 										//!< Position of incoming line of specified node
	int out, 									//!< Position of outgoing line of specified node
	DOUBLE *angle								//!< Turn angle returned
	);

//! Is turn attributes updated.
//!
//! @return 1 if yes, 0 if no or error code < 0.
int MnetworkTurnIsUpdated (
	MNETWORKHANDLE handle					//!< Handle returned by MnetworkMainInit function  
	);

//! Apply angle impedance array to entire network as turn impedance.
int MnetworkAngleApply (
	MNETWORKHANDLE handle					//!< Handle returned by MnetworkMainInit function  
	);

//! Is angle impedance array updated.
//!
//! @return 1 if yes, 0 if no or error code < 0.
int MnetworkAngleIsUpdated (
	MNETWORKHANDLE handle					//!< Handle returned by MnetworkMainInit function  
	);

//! Set impedance for given turn.
int MnetworkAngleSetImpedance (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	int angle, 									//!< Turn angle from range [-179,+180]
	DOUBLE impedance							//!< Impedance for given turn angle
	);

//! Get impedance for given turn.
int MnetworkAngleGetImpedance (
	MNETWORKHANDLE handle, 					//!< Handle returned by MnetworkMainInit function  
	int angle, 									//!< Turn angle from range [-179,+180]
	DOUBLE *impedance							//!< Impedance for given turn angle
	);

// Stop functions

//! Calculate route for given stop array.
int MnetworkStopCalculateRoute (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	MNETWORKROUTEHANDLE *routeHandle		//!< Handle that represents result of routing
	);

//! Clone given stop array.
int MnetworkStopClone (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	MNETWORKSTOPHANDLE *cloneStopHandle	//!< Clone handle   
	);

//! Free stop array handle.
int MnetworkStopFree (						
	MNETWORKSTOPHANDLE *stopHandle		//!< Handle returned by MnetworkMainCreateStop function  
	);

//! Is stop array updated.
//!
//! @return 1 if yes, 0 if no or error code < 0.
int MnetworkStopIsUpdated (
	MNETWORKSTOPHANDLE stopHandle			//!< Handle returned by MnetworkMainCreateStop function  
	);

//! Is there "stop" table name.
//!
//! @return 1 if yes, 0 if no or error code < 0/ 
int MnetworkStopIsTable (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	const RVC::OBJECTNAME& tablename						//!< Table name 
	);
	
//! Get array of database table names that contains saved stop array information. 
int MnetworkStopGetTableNames (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	RVC::OBJECTNAME **tablename, 				//!< "Stop" tables names array allocated and returned
	INT32 *tablecount							//!< Number of names in array
	);
	
//! Open "stop" table. 
int MnetworkStopOpenTable (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	const RVC::OBJECTNAME& tablename						//!< Table name to open 
	);

//! Save "stop" table.
int MnetworkStopSaveTable (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	RVC::OBJECTNAME& tablename						//!< Table name to save 
	);
	 
//! Add node to stop array to specified position.
int MnetworkStopAdd (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	INT32 node, 								//!< Node value
	int position								//!< Position to add
	);

//! Delete stops at specified positions.
int MnetworkStopDelete (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	int *positions, 							//!< Array of positions
	int numPositions							//!< Number of positions in array
	);

//! Delete all stops.
int MnetworkStopDeleteAll (
	MNETWORKSTOPHANDLE stopHandle			//!< Handle returned by MnetworkMainCreateStop function  
	);

//! Move stop to specified position.
int MnetworkStopMove (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	int oldPosition, 							//!< Old position
	int newPosition							//!< New position
	);

//! Get stop at specified position as node value from array of nodes.
//!
//! @return Node or error code < 0. 
INT32 MnetworkStopGet (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	int position								//!< Position of stop
	);

//! Get number of stops.
int MnetworkStopGetNumber (
	MNETWORKSTOPHANDLE stopHandle			//!< Handle returned by MnetworkMainCreateStop function  
	);

//! Get stop array as nodes.
int MnetworkStopGetAll (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	INT32 **nodes, 							//!< Nodes array allocated and returned
	INT32 *numNodes							//!< Number of nodes in array
	);

//! Set demand of stop at specified position.
int MnetworkStopSetDemand (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	int position, 								//!< Position of stop
	DOUBLE demand								//!< Demand value
	);

//! Get demand of stop at specified position.
int MnetworkStopGetDemand (
	MNETWORKSTOPHANDLE stopHandle, 		//!< Handle returned by MnetworkMainCreateStop function  
	int position, 								//!< Position of stop
	DOUBLE *demand								//!< Demand value
	);


// MultiRoute functions

//! Calculate multiroutes from source node to all destination nodes.
int MnetworkCalculateMultiRoute (
	MNETWORKHANDLE handle,							  //!< Handle returned by MnetworkMainInit function
	INT32 source,										  //!< Source node
	INT32 *destinations,								  //!< Destination nodes array
	INT32 numDestinations,							  //!< Number of destination nodes
	MNETWORKROUTEHANDLE *multiRouteHandle		  //!< Handle that represents result of multirouting
	);

int MnetworkMultiRouteGetRoute (
	MNETWORKMULTIROUTEHANDLE multiRouteHandle,  //!< Handle returned by MnetworkCalculateMultiRoute function
	INT32 destination,								  //!< Destination node
	MNETWORKROUTEHANDLE *routeHandle				  //!< Handle that represents result of routing
	);

//! Free multiroute handle.
int MnetworkMultiRouteFree (
	MNETWORKMULTIROUTEHANDLE *multiRouteHandle	//!< Handle returned by MnetworkCalculateMultiRoute function
	);

// Route functions

//! Restore stop handle that was used to generate given route.
int MnetworkRouteRestoreStop (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	MNETWORKSTOPHANDLE *stopHandle		//!< Handle returned by MnetworkMainCreateStop function  
	);

//! Free route handle.
int MnetworkRouteFree (
	MNETWORKROUTEHANDLE *routeHandle		//!< Handle returned by MnetworkStopCalculateRoute function  
	);

//! Get number of nodes in route.
//!
//! @return Number of nodes or error code < 0.
INT32 MnetworkRouteGetNumberOfNodes (
	MNETWORKROUTEHANDLE routeHandle
	);

//! Get node at specified position.
//!
//! @return Node or error code < 0.
INT32 MnetworkRouteGetNode (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	int position								//!< Position of node in route
	);

//! Get number of lines in route.
//!
//! @return Number of lines or error code < 0.
INT32 MnetworkRouteGetNumberOfLines (
	MNETWORKROUTEHANDLE routeHandle		//!< Handle returned by MnetworkStopCalculateRoute function  
	);

//! Get line at specified position.
//! 
//! @return Line or error code < 0.
INT32 MnetworkRouteGetLine (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	int position								//!< Position of line in route
	);

//! Position of line in route.
//!
//! @return 1 if yes, 0 if no or error code < 0.
int MnetworkRouteIsNodeStop (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	int position								//!< Position of node in route
	);

//! Is node at specified position a turn.
//!
//! @return 1 if yes, 0 if no or error code < 0.
int MnetworkRouteIsNodeTurn (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	int position								//!< Position of node in route
	);

//! Get direction of line at specified position.
//!
//! @return Direction or error code < 0.
//!
//! Direction on line:			
//!		NETWORK_LINE_DIRECTION_FromTo		 
//!		NETWORK_LINE_DIRECTION_ToFrom	
int MnetworkRouteGetLineDirection (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	int position								//!< Position of line in route
	);

//! Position of line in route.
int MnetworkRouteGetResultAsNodes (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	INT32 **nodes, 							//!< Nodes array allocated and returned
	INT32 *numNodes							//!< Number of nodes in array
	);

//! Get route as lines array.
int MnetworkRouteGetResultAsLines (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	INT32 **lines, 							//!< Lines array allocated and returned
	INT32 *numLines							//!< Number of lines in array
	);

//! Get route as 2D point array in object coordinates.
int MnetworkRouteGetResultAsPoints (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	DPOINT2D **points, 						//!< 2D points array allocated and returned
	INT32 *numPoints							//!< Number of points in array
	);

//! Get simple report.
int MnetworkRouteGetReport (
	MNETWORKROUTEHANDLE routeHandle, 	//!< Handle returned by MnetworkStopCalculateRoute function  
	MIUNICODE **report							//!< Simple report allocated and returned
	);

// Center functions

//! Calculate allocation for given center array.
//!
//! Type of allocation: 				
//!		NETWORK_ALLOCATION_In				
//!		NETWORK_ALLOCATION_Out
int MnetworkCenterCalculateAllocation (
	MNETWORKCENTERHANDLE centerHandle, 				//!< Handle returned by MnetworkMainCreateCenter function
	int type, 												//!< Type of allocation
	MNETWORKALLOCATIONHANDLE *allocationHandle	//!< Handle that represents result of allocation		
	);

//! Clone given center array.
int MnetworkCenterClone (
	MNETWORKCENTERHANDLE centerHandle, 				//!< Handle returned by MnetworkMainCreateCenter function	
	MNETWORKCENTERHANDLE *cloneCenterHandle		//!< Clone handle 	
	);

//! Free center array handle.
int MnetworkCenterFree (
	MNETWORKCENTERHANDLE *centerHandle	//!< Handle returned by MnetworkMainCreateCenter function	
	);

//! Is centers array updated.
//!
//! @return 1 if yes, 0 if no or error code < 0.
int MnetworkCenterIsUpdated (
	MNETWORKCENTERHANDLE centerHandle	//!< Handle returned by MnetworkMainCreateCenter function	
	); 

//! Is there "center" table name.
//!
//! @return 1 if yes, 0 if no or error code < 0.
int MnetworkCenterIsTable (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	const RVC::OBJECTNAME& tablename						//!< Table name 
	);
	
//! Get array of database table names that contains saved center array information. 
int MnetworkCenterGetTableNames (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	RVC::OBJECTNAME **tablename, 				//!< "Center" tables names array allocated and returned
	INT32 *tablecount							//!< Number of names in array
	);
	
//! Open "center" table. 
int MnetworkCenterOpenTable (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function
	const RVC::OBJECTNAME& tablename						//!< Table name to open 
	);

//!	Save "center" table.
int MnetworkCenterSaveTable (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	RVC::OBJECTNAME& tablename						//!< Table name to save 
	); 

//! Add node to center array.
int MnetworkCenterAdd (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	INT32 node									//!< Node value
	);

//! Delete centers at specified positions.
int MnetworkCenterDelete (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int *positions, 							//!< Array of positions
	int numPositions							//!< Number of positions in array
	);

//! Delete all centers.
int MnetworkCenterDeleteAll (
	MNETWORKCENTERHANDLE centerHandle	//!< Handle returned by MnetworkMainCreateCenter function	
	);

//!	Get center at specified position as node value from array of centers.
//!
//! @return Node value or error code < 0.
INT32 MnetworkCenterGet (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position								//!< Position of the center
	);

//! Get position of node in array of centers.
//!
//! @return Position or error code < 0.
int MnetworkCenterGetPosition (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	INT32 node									//!< Node value
	);

//! Get number of centers.
//!
//! @return Number of centers or error code < 0.
int MnetworkCenterGetNumber (
	MNETWORKCENTERHANDLE centerHandle	//!< Handle returned by MnetworkMainCreateCenter function	
	);

//! Get center array as nodes.
int MnetworkCenterGetAll (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	INT32 **nodes, 							//!< Nodes array allocated and returned
	INT32 *numNodes							//!< Number of nodes in array
	);

//! Set capacity of center at specified position.
int MnetworkCenterSetCapacity (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position, 								//!< Position of the center
	DOUBLE capacity							//!< Capacity value to set
	);

//! Get capacity of center at specified position.
int MnetworkCenterGetCapacity (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position, 								//!< Position of the center
	DOUBLE *capacity							//!< Capacity value returned
	);

//! Set impedance limit of center at specified position.
int MnetworkCenterSetImpedanceLimit (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position, 								//!< Position of the center
	DOUBLE limit								//!< Impedance limit value to set
	);

//! Get impedance limit of center at specified position.
int MnetworkCenterGetImpedanceLimit (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position, 								//!< Position of the center
	DOUBLE *limit								//!< Impedance limit value returned
	);

//! Set impedance delay of center at specified position.
int MnetworkCenterSetImpedanceDelay (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position, 								//!< Position of the center
	DOUBLE delay								//!< Impedance delay value to set
	);

//! Get impedance delay of center at specified position.
int MnetworkCenterGetImpedanceDelay (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position, 								//!< Position of the center
	DOUBLE *delay								//!< Impedance delay value returned
	);

//! Set color of center at specified position.
int MnetworkCenterSetColor (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position, 								//!< Position of the center
	COLOR *color								//!< Color value to set
	);

//! Get color of center at specified position.
int MnetworkCenterGetColor (
	MNETWORKCENTERHANDLE centerHandle, 	//!< Handle returned by MnetworkMainCreateCenter function	
	int position, 								//!< Position of the center
	COLOR *color								//!< Color value returned
	);

// Allocation functions

//! Restore center handle that was used to generate given allocation.
int MnetworkAllocationRestoreCenter (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function
	MNETWORKCENTERHANDLE *centerHandle				//!< Handle returned by MnetworkMainCreateCenter function
	);

//! Free allocation handle.
int MnetworkAllocationFree (
	MNETWORKALLOCATIONHANDLE *allocationHandle	//!< Handle returned by MnetworkCenterCalculateAllocation function
	);

//! Get array of positions of allocated centers for every line.
int MnetworkAllocationGetResultAsPositions (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  
	INT32 **positions, 									//!< Positions array
	INT32 *numPositions									//!< Size of positions array
	);

//! Get type of allocation.
//!
//! @return Type of allocation or error code < 0.
//!
//! Type of allocation: 				
//! 	NETWORK_ALLOCATION_In				
//!		NETWORK_ALLOCATION_Out
int MnetworkAllocationGetType (
	MNETWORKALLOCATIONHANDLE allocationHandle		//!< Handle returned by MnetworkCenterCalculateAllocation function  
	);

//! Get simple report.
int MnetworkAllocationGetReport (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  
	MIUNICODE **report										//!< Report as unicode string allocated and returned
	);

//! Get allocated center at specified position as node value from array of allocated centers.
//!
//! @return Node value or error code < 0.
INT32 MnetworkAllocatedCenterGet (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function
	int position											//!< Position of the allocated center
	);

//! Get number of allocated centers.
//!
//! @return Number of allocated centers or error code < 0.
INT32 MnetworkAllocatedCenterGetNumber (
	MNETWORKALLOCATIONHANDLE allocationHandle		//!< Handle returned by MnetworkCenterCalculateAllocation function  
	);

//! Get position of node in array of allocated centers.
//!
//! @return Position or error code < 0.
int MnetworkAllocatedCenterGetPosition (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function
	INT32 node												//!< Node value
	);

//! Get color of allocated center at specified position.
int MnetworkAllocatedCenterGetColor (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function 
	int position, 											//!< Position of the allocated center
	COLOR *color											//!< Color value returned
	);

//! Get number of lines assigned to allocated center at specified position.
//!
//! @return Number of lines assigned to allocated center or error code < 0.
INT32 MnetworkAllocatedCenterGetNumberOfLines (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  
	int position											//!< Position of the allocated center
	);

//! Get capacity of allocated center at specified position.
int MnetworkAllocatedCenterGetCapacity (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  		
	int position, 											//!< Position of the allocated center
	DOUBLE *capacity										//!< Capacity value returned
	);

//! Get demand of allocated center at specified position.
int MnetworkAllocatedCenterGetDemand (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  		
	int position, 											//!< Position of the allocated center		
	DOUBLE *demand											//!< Demand value returned
	);

//! Get impedance limit of allocated center at specified position.
int MnetworkAllocatedCenterGetImpedanceLimit (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  		
	int position, 											//!< Position of the allocated center		
	DOUBLE *limit											//!< Impedance limit value returned		
	);

//! Get impedance delay of allocated center at specified position.
int MnetworkAllocatedCenterGetImpedanceDelay (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  
	int position, 											//!< Position of the allocated center		
	DOUBLE *delay											//!< Impedance delay value returned		
	);

//! Get maximum impedance of allocated center at specified position.
int MnetworkAllocatedCenterGetMaximumImpedance (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  
	int position, 											//!< Position of the allocated center
	DOUBLE *maximum										//!< Maximum impedance value returned		
	);

//! Get average impedance of allocated center at specified position.
int MnetworkAllocatedCenterGetAverageImpedance (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  		
	int position, 											//!< Position of the allocated center		
	DOUBLE *average										//!< Average impedance value returned		
	);

//! Get number of lines that was used for allocation.
//!
//! @return Number of lines or error code < 0.
INT32 MnetworkAllocatedLineGetNumber (
	MNETWORKALLOCATIONHANDLE allocationHandle		//!< Handle returned by MnetworkCenterCalculateAllocation function  
	);

//! Get position of allocated center than assigned given line.
int MnetworkAllocatedLineGetPosition (
	MNETWORKALLOCATIONHANDLE allocationHandle, 	//!< Handle returned by MnetworkCenterCalculateAllocation function  
	INT32 line												//!< Line value
	);

//!@}

#if defined(__cplusplus)
}								
#endif

#endif	//!<  INC_MI32_MNETFUNC_H 
