/**
 * \file uom.h <mi32/uom.h>
 * \brief UOM (Unit of Measure) system definitions
 *
 * \if NODOC
 * $Id: uom.h_v 1.33 2012/09/26 20:24:46 mju Exp $
 *
 * $Log: uom.h_v $
 * Revision 1.33  2012/09/26 20:24:46  mju
 * Add unit_length_foot_us.
 *
 * Revision 1.32  2012/04/26 17:38:25  bcolombe
 * Add tenthfeet, hundredthfeet
 *
 * Revision 1.31  2012/03/15 15:30:37  bcolombe
 * Add decimeter
 *
 * Revision 1.30  2011/03/11 18:52:39  mju
 * Add unit::define.
 *
 * Revision 1.29  2011/03/11 17:30:39  mju
 * Add formatScale.
 *
 * Revision 1.28  2011/03/10 14:44:50  mju
 * Use libexport on convertToScale.
 *
 * Revision 1.27  2011/03/09 21:16:50  mju
 * Add convertToScale.
 *
 * Revision 1.26  2011/03/02 21:37:27  mju
 * Add idnum_length_mile.
 *
 * Revision 1.25  2011/02/28 17:55:38  mju
 * Add systemid enum.
 *
 * Revision 1.24  2011/01/12 17:56:39  mju
 * Add unit_length_mile.
 *
 * Revision 1.23  2010/12/08 21:12:52  mju
 * *** empty log message ***
 *
 * Revision 1.22  2007/07/06 21:24:26  mju
 * Add idnum_planeangle_arcminute.
 *
 * Revision 1.21  2007/05/04 19:55:57  mju
 * Remove identifer hdr.
 *
 * Revision 1.20  2007/05/02 16:29:38  mju
 * Deprecate inclusion if identifier hdr.
 *
 * Revision 1.19  2007/02/07 16:29:17  mju
 * Remove unused legacy system::findUnit variant.
 *
 * Revision 1.18  2006/11/30 22:36:48  mju
 * Exclude deprecated fn if no_deprecated defined.
 *
 * Revision 1.17  2006/10/12 15:45:10  scowan
 * Fwd decl mistring instead of including the file.
 *
 * Revision 1.16  2006/08/25 12:56:26  mju
 * Add unit_velocity and corresponding idnum values.
 *
 * Revision 1.15  2005/11/22 22:16:44  mju
 * Add duration and age unit types.
 *
 * Revision 1.14  2004/12/10 17:18:07  mju
 * Add constants for more planeangle unit ids.
 *
 * Revision 1.13  2004/07/20 15:46:29  mju
 * Add idnum for arcsecond.
 *
 * Revision 1.12  2004/07/19 20:57:04  mju
 * Add scale id nums.
 *
 * Revision 1.11  2004/06/25 15:12:43  mju
 * Add GetCompositeIdNum.
 *
 * Revision 1.10  2004/06/24 19:12:51  mju
 * Add IDNUM for composite measure/unit ids.
 * Allow unit construction/assign from idnum.
 *
 * Revision 1.9  2004/06/04 15:04:07  mju
 * Add UNIT_None.
 *
 * Revision 1.8  2004/03/18 22:08:07  mju
 * Add unit_scale_Unity.
 *
 * Revision 1.7  2003/09/29 16:57:51  mju
 * Add FindUnit variants using only idnum and id.
 *
 * Revision 1.6  2003/09/25 21:29:07  dwilliss
 * Don't capitalize enum any more. Genitor needed it, doxygen doesn't like it.
 *
 * Revision 1.5  2003/09/19 14:05:00  mju
 * Fix doc comments.
 *
 * Revision 1.4  2003/09/17 15:22:22  dwilliss
 * Fixed operator== and operator!= to return bool (was unspecified)
 *
 * Revision 1.3  2003/09/17 13:49:18  mju
 * Add ==/!= for unit.
 *
 * Revision 1.2  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  2003/09/05 17:09:12  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_UOM_H
#define	INC_MI32_UOM_H

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

#ifdef GEOMDLL
	#define LIBEXPORT MI_DLLEXPORT
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
class IDENTIFIER;
class IDENTIFIERLIST;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace UOM {

#ifndef GENERATING_DOXYGEN_OUTPUT
class SYSTEM;
class MEASURE;
class UNIT;
class CONVERTER;
class MEASURE_PRIV;
class UNIT_PRIV;
class CONVERTER_PRIV;
typedef MILIST<UNIT> UNITLIST;
#endif // GENERATING_DOXYGEN_OUTPUT

//-------------------------------------------------------------------------------------------------------------------
//! Measure container.
//! Normally one of the UOM::SYSTEM::FindMeasure() methods is used obtain a MEASURE instance.
class CLASSLIBEXPORT MEASURE {
	public:

		//! Enumeration of measure types.
		//! Invariant, may be saved in files by number.
		//! Same as codes used in MicroImages codespace.
		enum TYPE {
			TYPE_Area =							1,		//!< Area [m]
			TYPE_Length =						2,		//!< Length [meter, m]
			TYPE_Distance =					TYPE_Length,
			TYPE_PlaneAngle =					3,		//!< Plane angle [radian, rad]
			TYPE_Time =							4,		//!< Time [second, s]
			TYPE_Temperature =				5,		//!< Temperature [kelvin, K]
			TYPE_Volume =						6,		//!< Volume [m]
			TYPE_Mass =							7,		//!< Mass [kilogram, kg]
			TYPE_Current =						8,		//!< Electric current [ampere, A]
			TYPE_Amount =						9,		//!< Amount of substance [mole, mol]
			TYPE_LuminousIntensity =		10,	//!< Luminous intensity [candela, cd]
			TYPE_SolidAngle =					11,	//!< Solid angle [steradian, sr]
			TYPE_Velocity =					12,	//!< Velocity [m/s]
			TYPE_Acceleration =				13,	//!< Acceleration [m/s]
			TYPE_VolumetricFlowRate =		14,	//!< Volumetric flow rate [M/s]
			TYPE_MassFlowRate =				15,	//!< Mass flow rate [kg/s]
			TYPE_WaveNumber =					16,	//!< Wave number [1/m]
			TYPE_Density =						17,	//!< Density [kg/m]
			TYPE_SpecificVolume =			18,	//!< Specific volume [m/kg]
			TYPE_CurrentDensity =			19,	//!< Current density [A/m]
			TYPE_MagneticFieldStrength =	20,	//!< Magnetic field strength [A/m]
			TYPE_Concentration =				21,	//!< Concentration [mol/m]
			TYPE_Luminance =					22,	//!< Luminance [cd/m]
			TYPE_Frequency =					23,	//!< Frequency [hertz, Hz]
			TYPE_Force =						24,	//!< Force [newton, N]
			TYPE_Pressure =					25,	//!< Pressure, stress [pascal, Pa]
			TYPE_Energy =						26,	//!< Energy, work, quantity of heat [joule, J]
			TYPE_Power =						27,	//!< Power, radiant flux [watt, W]
			TYPE_ElectricCharge =			28,	//!< Electric charge [coulomb, C]
			TYPE_ElectricPotential =		29,	//!< Electric potential [volt, V]
			TYPE_Capacitance =				30,	//!< Capacitance [farad, F]
			TYPE_ElectricResistance =		31,	//!< Electric resistance [ohm, Omega]
			TYPE_ElectricConductance =		31,	//!< Electric conductance [siemens, S]
			TYPE_MagneticFlux =				33,	//!< Magentic flux [weber, Wb]
			TYPE_MagneticFluxDensity =		34,	//!< Magnetic flux density [tesla, T]
			TYPE_Inductance =					35,	//!< Inductance [henry, H]
			TYPE_LuminousFlux =				36,	//!< Luminous flux [lumen, lm]
			TYPE_RadionuclideActivity =	37,	//!< Activity of radionuclide [becquerel, Bq]
			TYPE_AbsorbedDose =				38,	//!< Absorbed dose [gray, Gy]
			TYPE_DoseEquivalent =			39,	//!< Dose equivalent [sievert, Sv]
			TYPE_DynamicViscosity =			40,	//!< Dynamic viscosity [Pas]
			TYPE_MomentOfForce =				41,	//!< Moment of force [Nm]
			TYPE_SurfaceTension =			42,	//!< Surface tension [N/m]
			TYPE_Irradiance =					43,	//!< Irradiance, heat flux density [W/m]
			TYPE_Entropy =						44,	//!< Entropy, heat capacity [J/K]
			TYPE_SpecificHeatCapacity =	45,	//!< Specific heat capacity [J/(kgK)]
			TYPE_SpecificEnergy =			46,	//!< Specific energy [J/kg]
			TYPE_ThermalConductivity =		47,	//!< Thermal conductivity [W/(mK)]
			TYPE_EnergyDensity =				48,	//!< Energy density [J/m]
			TYPE_ElectricFieldStrength =	49,	//!< Electric field strength [V/m]
			TYPE_ElectricChargeDensity =	50,	//!< Electric charge density [C/m]
			TYPE_ElectricFluxDensity =		51,	//!< Electric flux density [C/m]
			TYPE_Permitivity =				52,	//!< Permitivity [F/m]
			TYPE_Permeability =				53,	//!< Permeability [H/m]
			TYPE_MolarEnergy =				54,	//!< Molar energy [J/mol]
			TYPE_MolarEntropy =				55,	//!< Molar entropy [J/(molK)]
			TYPE_Exposure =					56,	//!< Exposure (X and gamma rays) [C/kg]
			TYPE_AbsorbedDoseRate =			57,	//!< Absorbed dose rate [Gy/s]
			TYPE_AngularVelocity =			58,	//!< Angular velocity [rad/s]
			TYPE_AngularAcceleration =		59,	//!< Angular acceleration [rad/s]
			TYPE_RadiantIntensity =			60,	//!< Radiant intensity [W/sr]
			TYPE_Radiance =					61,	//!< Radiance [W/msr]
			TYPE_Scale =						62,	//!< Scale (unit/unit)
			TYPE_Image =						63,	//!< Image (pixel)
			TYPE_Duration =					64,	//!< Duration (years, y)
			TYPE_Age =							65,	//!< Age (annum, a)

			TYPE_LASTNONDATE =				65,	//!< Last measure type other than date

			TYPE_Date =							99,	//!< Date, possibly formatted
			TYPE_None =							255
			};

		//! Default constructor, constructs instance with TYPE_None.
		MEASURE (
			);

		//! Copy constructor.
		MEASURE (
			const MEASURE& rhs
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Internal constructor.
		MEASURE (
			MEASURE_PRIV *pPriv
			);
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Destructor.
		~MEASURE (
			);

		//! Assignment.
		MEASURE& operator= (
			const MEASURE& rhs
			);

		//! Find unit using MicroImages identification code.
		//! @return true if found, false if not.
		bool FindUnit (
			int micode,								//!< MicroImages identification code
			UNIT& unitret							//!< Unit returned if found
			) const;

		//! Find unit using IDENTIFIER.
		//! @return true if found, false if not.
		bool FindUnit (
			const IDENTIFIER& identifier,		//!< Identifier
			UNIT& unitret							//!< Unit returned if found
			) const;

		//! Find unit using name.
		//! Searches all 'string' identifiers, localized names, localized symbols.
		//! @return true if found, false if not.
		bool FindUnit (
			const MISTRING& name,				//!< Unit name
			UNIT& unitret							//!< Unit returned if found
			) const;
			
		//! Get base unit for measure.
		//! @return Reference to base unit.
		const UNIT& GetBaseUnit (
			) const;

		//! Get identifer list for measure.
		const IDENTIFIERLIST& GetIdList (
			) const;

		//! Get name of measure.
		const MISTRING& GetName (
			) const;

		//! Get measure type enumeration.
		TYPE GetType (
			) const;

		//! Get unit list for measure.
		const UNITLIST& GetUnitList (
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MEASURE_PRIV *m_pPriv;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


//! Equality.
inline bool operator== (
	const MEASURE& lhs,
	const MEASURE& rhs
	) { return (lhs.GetType() == rhs.GetType()); }

//! Inequality.
inline bool operator!= (
	const MEASURE& lhs,
	const MEASURE& rhs
	) { return (lhs.GetType() != rhs.GetType()); }

/*
#if defined(WIN32) && !defined(GEOMDLL) && !defined(NODLLLINKAGE)
#pragma warning(disable:4231)
extern template class CLASSLIBEXPORT MILIST<MEASURE>;		//! Template instantiation
#pragma warning(default:4231)
#endif
*/
typedef MILIST<MEASURE> MEASURELIST;

//-------------------------------------------------------------------------------------------------------------------
//!		Commmon MicroImages numeric Unit identifiers

#define	UNIT_None							0

#define	UNIT_Scale_Unity					0
#define	UNIT_Scale_Percent				1
#define	UNIT_Scale_PartsPerMillion		3

#define	UNIT_Area_SquareMeter			0
#define	UNIT_Area_SquareMeters			0
#define	UNIT_Area_SquareKilometer		3
#define	UNIT_Area_SquareKilometers		3
#define	UNIT_Area_SquareMillimeter		2
#define	UNIT_Area_SquareMillimeters	2

#define	UNIT_Length_Meter				0
#define	UNIT_Length_Kilometer		3
#define	UNIT_Length_Decimeter		10
#define	UNIT_Length_Centimeter		1
#define	UNIT_Length_Millimeter		2
#define	UNIT_Length_Micrometer		8
#define	UNIT_Length_Nanometer		9
#define	UNIT_Length_Foot				4
#define	UNIT_Length_TenthFoot		64
#define	UNIT_Length_HundredthFoot	65
#define	UNIT_Length_Inch				6
#define	UNIT_Length_Mile				7
#define	UNIT_Length_Foot_US			16

#define	UNIT_Distance_Meters				UNIT_Length_Meter
#define	UNIT_Distance_Kilometers		UNIT_Length_Kilometer
#define	UNIT_Distance_Centimeters		UNIT_Length_Centimeter
#define	UNIT_Distance_Millimeters		UNIT_Length_Millimeter
#define	UNIT_Distance_Micrometers		UNIT_Length_Micrometer
#define	UNIT_Distance_Nanometers		UNIT_Length_Nanometer
#define	UNIT_Distance_Feet				UNIT_Length_Foot
#define	UNIT_Distance_Inches				UNIT_Length_Inch

#define	UNIT_PlaneAngle_Degree					0
#define	UNIT_PlaneAngle_Degrees					0
#define	UNIT_PlaneAngle_Radian					1
#define	UNIT_PlaneAngle_Radians					1
#define	UNIT_PlaneAngle_Grad						3
#define	UNIT_PlaneAngle_ArcSecond				4
#define	UNIT_PlaneAngle_ArcMinute				5
#define	UNIT_PlaneAngle_Gon						6
#define	UNIT_PlaneAngle_CentisimalMinute		8
#define	UNIT_PlaneAngle_CentisimalSecond		9
#define	UNIT_PlaneAngle_Hour						10

#define	UNIT_Time_Second					0
#define	UNIT_Time_Seconds					0
#define	UNIT_Time_Minute					1
#define	UNIT_Time_Hour						2
#define	UNIT_Time_Millisecond			6
#define	UNIT_Time_Milliseconds			6
#define	UNIT_Time_Microsecond			7
#define	UNIT_Time_Microseconds			7

#define	UNIT_Volume_CubicMeter			0
#define	UNIT_Volume_CubicMeters			0

#define	UNIT_Velocity_MetersPerSecond		0
#define	UNIT_Velocity_FeetPerSecond		2
#define	UNIT_Velocity_KilometersPerHour	4
#define	UNIT_Velocity_MilesPerHour			5
#define	UNIT_Velocity_Knots					6

//---------------------------------------------------------------------------------------------------------------------

//! Unit system IDs.
//! These are defined primarily to allow applications to provide a user interface for selection of a system of units.
//! There is no specific set of units defined as belonging to a particular system.
enum SYSTEMID {
	SYSTEMID_Unspecified =		0,
	SYSTEMID_Metric =				1,		//!< Metric/SI (meter, kilometer)
	SYSTEMID_English =			2,		//!< (foot, mile)
	};

//---------------------------------------------------------------------------------------------------------------------

//! Composite measure/unit ID enumeration.
enum IDNUM {
	IDNUM_Unspecified =					0,

	IDNUM_Scale_Unity =					(MEASURE::TYPE_Scale << 8) | UNIT_Scale_Unity,
	IDNUM_Scale_Percent =				(MEASURE::TYPE_Scale << 8) | UNIT_Scale_Percent,
	IDNUM_Scale_PartsPerMillion =		(MEASURE::TYPE_Scale << 8) | UNIT_Scale_PartsPerMillion,

	IDNUM_Length_Kilometer =			(MEASURE::TYPE_Length << 8) | UNIT_Length_Kilometer,
	IDNUM_Length_Decimeter =			(MEASURE::TYPE_Length << 8) | UNIT_Length_Decimeter,
	IDNUM_Length_Meter =				(MEASURE::TYPE_Length << 8) | UNIT_Length_Meter,
	IDNUM_Length_Centimeter =			(MEASURE::TYPE_Length << 8) | UNIT_Length_Centimeter,
	IDNUM_Length_Millimeter =			(MEASURE::TYPE_Length << 8) | UNIT_Length_Millimeter,
	IDNUM_Length_Micrometer =			(MEASURE::TYPE_Length << 8) | UNIT_Length_Micrometer,
	IDNUM_Length_Nanometer =			(MEASURE::TYPE_Length << 8) | UNIT_Length_Nanometer,
	IDNUM_Length_Mile =					(MEASURE::TYPE_Length << 8) | UNIT_Length_Mile,
	IDNUM_Length_Foot =					(MEASURE::TYPE_Length << 8) | UNIT_Length_Foot,
	IDNUM_Length_TenthFoot =			(MEASURE::TYPE_Length << 8) | UNIT_Length_TenthFoot,
	IDNUM_Length_HundredthFoot =		(MEASURE::TYPE_Length << 8) | UNIT_Length_HundredthFoot,
	IDNUM_Length_Inch =					(MEASURE::TYPE_Length << 8) | UNIT_Length_Inch,

	IDNUM_PlaneAngle_Degree =			(MEASURE::TYPE_PlaneAngle << 8) | UNIT_PlaneAngle_Degree,
	IDNUM_PlaneAngle_Radian =			(MEASURE::TYPE_PlaneAngle << 8) | UNIT_PlaneAngle_Radian,
	IDNUM_PlaneAngle_ArcMinute =		(MEASURE::TYPE_PlaneAngle << 8) | UNIT_PlaneAngle_ArcMinute,
	IDNUM_PlaneAngle_ArcSecond =		(MEASURE::TYPE_PlaneAngle << 8) | UNIT_PlaneAngle_ArcSecond,

	IDNUM_Area_SquareMeter =			(MEASURE::TYPE_Area << 8) | UNIT_Area_SquareMeter,
	IDNUM_Area_SquareKilometer =		(MEASURE::TYPE_Area << 8) | UNIT_Area_SquareKilometer,
	IDNUM_Area_SquareMillimeter =		(MEASURE::TYPE_Area << 8) | UNIT_Area_SquareMillimeter,

	IDNUM_Time_Second =					(MEASURE::TYPE_Time << 8) | UNIT_Time_Second,
	IDNUM_Time_Millisecond =			(MEASURE::TYPE_Time << 8) | UNIT_Time_Millisecond,
	IDNUM_Time_Microsecond =			(MEASURE::TYPE_Time << 8) | UNIT_Time_Microsecond,

	IDNUM_Velocity_MetersPerSecond =	(MEASURE::TYPE_Velocity << 8) | UNIT_Velocity_MetersPerSecond,
	IDNUM_Velocity_FeetPerSecond =	(MEASURE::TYPE_Velocity << 8) | UNIT_Velocity_FeetPerSecond,
	IDNUM_Velocity_KilometersPerHour =	(MEASURE::TYPE_Velocity << 8) | UNIT_Velocity_KilometersPerHour,
	IDNUM_Velocity_MilesPerHour =		(MEASURE::TYPE_Velocity << 8) | UNIT_Velocity_MilesPerHour,
	IDNUM_Velocity_Knots =				(MEASURE::TYPE_Velocity << 8) | UNIT_Velocity_Knots,
	};

//-------------------------------------------------------------------------------------------------------------------

//! Single unit container.
//! Normally a UNIT instance is obtained using one of the MEASURE::FindUnit() or GetBaseUnit() methods.
class CLASSLIBEXPORT UNIT {
	public:

		enum NAMEUSAGE {
			NAMEUSAGE_Singular,
			NAMEUSAGE_Plural
			};

		//! Default constructor.
		UNIT (
			);

		//! Copy constructor.
		UNIT (
			const UNIT& rhs
			);

		//! Construct from composite measure/unit ID.
		UNIT (
			IDNUM idnum
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Internal constructor.
		UNIT (
			UNIT_PRIV *pPriv
			);
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Destructor.
		~UNIT (
			);

		//! Assignment.
		UNIT& operator= (
			const UNIT& rhs
			);

		//! Assignment from composite measure/unit ID.
		UNIT& operator= (
			IDNUM idnum
			);

		//! Convert value in base unit to this unit (numeric).
		double ConvertFromBase (
			double inval
			) const;

		//! Convert value in base unit to this unit (string).
		void ConvertFromBase (
			double inval,
			MISTRING& outstr
			) const;

		//! Convert numeric value in this unit to base unit.
		double ConvertToBase (
			double inval
			) const;

		//! Convert string value in this unit to base unit.
		double ConvertToBase (
			const MISTRING& instr
			) const;

		//! Define user-defined unit.
		void Define (
			const MEASURE::TYPE MeasureType,
			double ConversionToBase,
			const MISTRING& Symbol,
			const MISTRING& NameSingular,
			const MISTRING& NamePlural
			);

		// Get composite measure/unit IDNUM.
		IDNUM GetCompositeIdNum (
			) const;

		//! Get factor A in conversion formula.
		double GetFactorA (
			) const;

		//! Get factor B in conversion formula.
		double GetFactorB (
			) const;

		//! Get factor C in conversion formula.
		double GetFactorC (
			) const;

		//! Get identifer list for unit.
		const IDENTIFIERLIST& GetIdList (
			) const;

		//! Get MicroImages unit identification number.
		int GetIdNum (
			) const;

		//! Get measure unit applies to.
		const MEASURE& GetMeasure (
			) const;

		//! Get unit name.
		const MISTRING& GetName (
			NAMEUSAGE nameusage					//!< Usage name is for
			) const;

		//! Get unit symbol.
		const MISTRING& GetSymbol (
			) const;

		//! Determine if equal.
		bool IsEqual (
			const UNIT& rhs
			) const;

		//! Determine if unit requires a string representation.
		//! @return true if string, false if not.
		bool IsString (
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UNIT_PRIV *m_pPriv;
		friend class MEASURE_PRIV;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//! Equality.
inline bool operator== (
	const UNIT& lhs,
	const UNIT& rhs
	) { return (lhs.IsEqual(rhs)); }

//! Inequality.
inline bool operator!= (
	const UNIT& lhs,
	const UNIT& rhs
	) { return (!lhs.IsEqual(rhs)); }


//-------------------------------------------------------------------------------------------------------------------

//! System methods.
class CLASSLIBEXPORT SYSTEM {
	public:

		//! Find measure instance by IDENTIFIER.
		//! @return true if found, false if not.
		static bool FindMeasure (
			const IDENTIFIER& identifier,		//!< Identifier
			MEASURE& measure						//!< Measure returned if found
			);

		//! Find measure instance by TYPE.
		//! @return true if found, false if not.
		static bool FindMeasure (
			MEASURE::TYPE type,					//!< Measure type
			MEASURE& measure						//!< Measure returned if found
			);

		//! Find unit using MicroImages composite measure/unit identification number.
		//! Code is in the form 256 * MeasureNum + UnitNum;
		//! @return true if found, false if not.
		static bool FindUnit (
			IDNUM idnum,							//!< MicroImages identification code
			UNIT& unitret							//!< Unit returned if found
			);

		//! Find unit using TYPE and MicroImages unit identification number.
		//! @return true if found, false if not.
		static bool FindUnit (
			MEASURE::TYPE type,					//!< Measure type
			int micode,								//!< MicroImages identification code
			UNIT& unitret							//!< Unit returned if found
			);
			
		//! Find unit using IDENTIFIER.
		//! @return true if found, false if not.
		static bool FindUnit (
			const IDENTIFIER& identifier,		//!< Identifier
			UNIT& unitret							//!< Unit returned if found
			);
			
		//! Find unit using TYPE and IDENTIFIER.
		//! @return true if found, false if not.
		static bool FindUnit (
			MEASURE::TYPE type,					//!< Measure type
			const IDENTIFIER& identifier,		//!< Identifier
			UNIT& unitret							//!< Unit returned if found
			);
			
		//! Find unit using TYPE and name.
		//! Searches all 'string' identifiers, localized names, localized symbols.
		//! @return true if found, false if not.
		static bool FindUnit (
			MEASURE::TYPE type,					//!< Measure type
			const MISTRING& name,				//!< Unit name
			UNIT& unitret							//!< Unit returned if found
			);
			
		//! Get list of MEASUREs.
		//! Will not include entry for TYPE_None.
		static const MEASURELIST& GetMeasureList (
			);

		//! Initialize unit of measure system.
		//! Automatically called by application initialization code.
		static ERRVALUE Initialize ();
	};


//-------------------------------------------------------------------------------------------------------------------

class CLASSLIBEXPORT CONVERTER {
	public:

		//! Default constructor.
		CONVERTER (
			);

		//! Copy constructor.
		CONVERTER (
			const CONVERTER& rhs
			);

		//! Identity constructor.
		CONVERTER (
			const MEASURE& measure
			);

		//! Destructor.
		~CONVERTER ();

		//! Assignment.
		CONVERTER& operator= (
			const CONVERTER& rhs
			);

		//! Convert numeric value from source unit to target unit.
		//! If either source or target unit requires a string representation the original value will be returned.
		//! @return Converted value.
		double ConvertForward (
			double value							//!< Value in source unit to convert
			) const;

		//! Convert string in source unit to numeric value in target unit.
		//! If target unit requires a string representation an error will be returned.
		ERRVALUE ConvertForward (
			const MISTRING& instr,				//!< String in source unit to convert
			double& outval							//!< Value in target unit returned
			) const;
			
		//! Convert numeric value in source unit to string in target unit.
		//! If source unit requires a string representation an error will be returned.
		ERRVALUE ConvertForward (
			double inval,							//!< Value in source unit to convert
			MISTRING& outstr						//!< String in target unit returned
			) const;
			
		//! Convert string in source unit to string in target unit.
		ERRVALUE ConvertForward (
			const MISTRING& instr,				//!< String in source unit to convert
			MISTRING& outstr						//!< String in target unit returned
			) const;
			
		//! Convert numeric value from target unit to source unit.
		//! If either target or source unit requires a string representation the original value will be returned.
		//! @return Converted value.
		double ConvertInverse (
			double value							//!< Value in target unit to convert
			) const;

		//! Convert string in target unit to numeric value in source unit.
		//! If source unit requires a string representation an error will be returned.
		ERRVALUE ConvertInverse (
			const MISTRING& instr,				//!< String in target unit to convert
			double& outval							//!< Value in source unit returned
			) const;
			
		//! Convert numeric value in target unit to string in source unit.
		//! If target unit requires a string representation an error will be returned.
		ERRVALUE ConvertInverse (
			double inval,							//!< Value in target unit to convert
			MISTRING& outstr						//!< String in source unit returned
			) const;
			
		//! Convert string in target unit to string in source unit.
		ERRVALUE ConvertInverse (
			const MISTRING& instr,				//!< String in target unit to convert
			MISTRING& outstr						//!< String in source unit returned
			) const;
			
		//! Get measure associated with converter.			
		const MEASURE& GetMeasure (
			) const;

		//! Get scale multiplication value from source to target.
		double GetScale (
			) const;

		//! Get source unit.
		const UNIT& GetSourceUnit (
			) const;

		//! Get target unit.
		const UNIT& GetTargetUnit (
			) const;

		//! Read source unit from INI file.
		//! Target unit will be left unchanged.
		void IniReadSource (
			INIHANDLE IniHandle,
			const char *IniGroup,
			const char *IniName,
			int micode = 0								//!< Default unit code
			);

		//! Read target unit from INI file.
		//! Source unit will be left unchanged.
		void IniReadTarget (
			INIHANDLE IniHandle,
			const char *IniGroup,
			const char *IniName,
			int micode = 0								//!< Default unit code
			);

		//! Write source unit code to INI file.
		void IniWriteSource (
			INIHANDLE IniHandle,
			const char *IniGroup,
			const char *IniName
			) const;

		//! Write target unit code to INI file.
		void IniWriteTarget (
			INIHANDLE IniHandle,
			const char *IniGroup,
			const char *IniName
			) const;

		//! Set source unit.
		//! If the current measure is of TYPE_None it will be set to the measure of the specified unit.
		//! Otherwise, an error will be returned if the unit does not refer to the current measure.
		ERRVALUE SetSourceUnit (
			const UNIT& SourceUnit
			);

		//! Set target unit.
		//! If the current measure is of TYPE_None it will be set to the measure of the specified unit.
		//! Otherwise, an error will be returned if the unit does not refer to the current measure.
		ERRVALUE SetTargetUnit (
			const UNIT& TargetUnit
			);

		//! Set both source and target units.
		//! An error will be returned if the specified units do not refer to the same MEASURE.
		ERRVALUE SetUnits (
			const UNIT& SourceUnit,
			const UNIT& TargetUnit
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CONVERTER_PRIV *m_pPriv;
		void GetExclusive();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------
//		Functions

//! Convert scale denominator to corresponding value in unit system.
//! @return Converted value, typically in range from 1-9999 where possible.
//! Typically the converted value will be displayed to the user and should be limited to appropriate precision.
LIBEXPORT double ConvertToScale (
	double ScaleDenominator,		//!< Scale denominator, eg 100000 for scale of 1 to 100000
	UOM::SYSTEMID UnitSystem,		//!< Unit system
	const UOM::UNIT& UnitSrc,		//!< Unit for base measurement, typically cm or inch and corresponding to UnitSystem
	UOM::UNIT& UnitTgt				//!< Scaled target units in specified system returned
	);

//! Format settings for UOM::FormatScale.
enum FORMATSCALE {
	FORMATSCALE_Ratio =				0x00,		// Format as ratio (eg, 1 cm : 1234 km) [default]
	FORMATSCALE_UnitFraction =		0x01,		// Format as unit fraction (eg 1234 km/cm)
	FORMATSCALE_AddNewline =		0x02,		// Add newline to end if any formatting done
	FORMATSCALE_AddSpacesAlways =	0x04,		// Always add spaces even if unitless (eg 1 : 20000)
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORMATSCALE)
#endif

//! Format scale for display.
//! @return Formatted string, will be empty if no formatting done.
LIBEXPORT MISTRING FormatScale (
	double ScaleDenominator,		//!< Scale denominator, eg 100000 for scale of 1 to 100000
	UOM::SYSTEMID UnitSystem,		//!< Unit system
	const UOM::UNIT& UnitSrc,		//!< Unit for base measurement, typically cm or inch and corresponding to UnitSystem
	FORMATSCALE format,				//!< Formatting
	int MinPrecision = 3				//!< Minimum precision (2-5)
	);

//---------------------------------------------------------------------------------------------------------------------
};	//! End namespace UOM


// Legacy definitions
typedef	UOM::MEASURE::TYPE	UNIT_TYPE;

#define	UNIT_TYPE_Constant						0		//!< Constants and unitless database fields
#define	UNIT_TYPE_Area								UOM::MEASURE::TYPE_Area
#define	UNIT_TYPE_Length							UOM::MEASURE::TYPE_Length
#define	UNIT_TYPE_Distance						UOM::MEASURE::TYPE_Distance
#define	UNIT_TYPE_PlaneAngle						UOM::MEASURE::TYPE_PlaneAngle
#define	UNIT_TYPE_Time								UOM::MEASURE::TYPE_Time
#define	UNIT_TYPE_Volume							UOM::MEASURE::TYPE_Volume
#define	UNIT_TYPE_Velocity						UOM::MEASURE::TYPE_Velocity
#define	UNIT_TYPE_Acceleration					UOM::MEASURE::TYPE_Acceleration
#define	UNIT_TYPE_AngularVelocity				UOM::MEASURE::TYPE_AngularVelocity
#define	UNIT_TYPE_Date								UOM::MEASURE::TYPE_Date
#define	UNIT_TYPE_None								UOM::MEASURE::TYPE_None


#undef	CLASSLIBEXPORT
#undef	LIBEXPORT

#endif	//!< INC_MI32_UOM_H
