/**
 * \file rvc/contrast.h 
 * \brief interface for the RVC::CONTRAST class.
 *
 * \if NODOC
 * $Id: contrast.h_v 1.6 2012/06/11 15:42:04 mju Exp $
 *
 * $Log: contrast.h_v $
 * Revision 1.6  2012/06/11 15:42:04  mju
 * Change filter_component to only need raster::component.
 *
 * Revision 1.5  2012/06/05 21:35:10  mju
 * Support specifying raster component.
 *
 * Revision 1.4  2007/09/06 17:18:41  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.3  2006/12/21 16:18:45  mju
 * Add computeTable.
 *
 * Revision 1.1  2006/11/13 17:31:06  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_CONTRAST_H
#define INC_RVC_CONTRAST_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_RVC_RASTER_H
	#include <rvc/raster.h>
#endif

#ifndef INC_MI32_CONTRASTPARM_H
	#include <mi32/contrastparm.h>
#endif


namespace RVC {

//=====================================================================================================================

//! Manage RVC::OBJTYPE_ContrastTable RVC objects.
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Raster
class CONTRAST : public RVC::SIMPLE {
	public:

		//! Filter for contrast object based on associated RASTER::COMPONENT.
		class FILTER_COMPONENT : public OBJECT::FILTER {
			public:

				//! Constructor.
				FILTER_COMPONENT (
					RASTER::COMPONENT component	//!< Raster component
					);

				//! Destructor
				virtual ~FILTER_COMPONENT ();
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				RVC::RASTER::COMPONENT m_RasterComponent;

				virtual bool Filter (const RVC::OBJECT& obj);
				#endif
			};

		//! Convert "image component" to raster component for specified IMAGE::CELLTYPE.
		static RVC::RASTER::COMPONENT ImageComponentToRasterComponent (
			RVC::IMAGE::CELLTYPE CellType,
			UINT8 ImageComponent
			);

		//! Convert raster component to "image component" for specified IMAGE::CELLTYPE.
		static UINT8 RasterComponentToImageComponent (
			RVC::IMAGE::CELLTYPE CellType,
			RVC::RASTER::COMPONENT RasterComponent
			);

		//! Default constructor.
		CONTRAST ();

		//! Copy constructor.
		CONTRAST (
			const RVC::CONTRAST& rhs
			);

		//! Destructor.
		virtual ~CONTRAST ();

		//! Assignment.
		RVC::CONTRAST& operator= (
			const RVC::CONTRAST& rhs
			);

		//! Compute contrast table based on raster component.
		ERRVALUE ComputeTable (
			RVC::RASTER& ObjRaster,
			RVC::RASTER::COMPONENT component = RVC::RASTER::COMPONENT_Default
			);
			
		//! Get the current contrast parameters.
		const CONTRASTPARM& GetContrastParm () const
			{ return (m_ContrastParm); }

		//! Get raster component.
		RVC::RASTER::COMPONENT GetRasterComponent () const
			{ return (m_RasterComponent); }

		//! Set the contrast parameters.
		void SetContrastParm (
			const CONTRASTPARM& ContrastParm
			);

		//! Set raster component to associate parameters with.
		//! Required for saving with multi-component (eg RGB) raster.
		void SetRasterComponent (
			RVC::IMAGE::CELLTYPE CellType,
			RVC::RASTER::COMPONENT RasterComponent
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		// Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		CONTRASTPARM m_ContrastParm;
		RVC::RASTER::COMPONENT m_RasterComponent;
		#endif

	};

//=====================================================================================================================

}		// End namespace RVC

#endif 	// INC_RVC_CONTRAST_H
