/**
 * \file rvc/dbform.h
 * \brief interface for the RVC::DBFORM class.
 *
 * \if NODOC
 * $Id: dbform.h_v 1.17 2011/12/07 22:29:09 mju Exp $
 *
 * $Log: dbform.h_v $
 * Revision 1.17  2011/12/07 22:29:09  mju
 * Remove redundant hdr.
 *
 * Revision 1.16  2008/09/11 15:57:46  mju
 * Add formfield::ctrltype and flags enums.
 *
 * Revision 1.15  2007/09/06 17:18:43  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.9  2005/01/19 23:38:52  scowan
 * Support of descriptor class.
 *
 * Revision 1.1  2000/09/29 19:44:46  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_DBFORM_H
#define INC_RVC_DBFORM_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_RVCDBASE_H
	#include <mi32/rvcdbase.h>
#endif

namespace RVC {

//! Manages RVC::OBJTYPE_DatabaseForm RVC objects.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_DatabaseForm
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_DatabaseForm
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Database
//!		- RVC::OBJTYPE_Database_Raster
//!		- RVC::OBJTYPE_Database_VectorPoint
//!		- RVC::OBJTYPE_Database_VectorLine
//!		- RVC::OBJTYPE_Database_VectorPolygon
//!		- RVC::OBJTYPE_Database_CAD
//!		- RVC::OBJTYPE_Database_TIN
//!
//!	Read(), Make() and Write() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods
class DBFORM : public RVC::SIMPLE {
	public:
	
		struct FORMFIELD;

		enum FORMFLAGS {
			FORMFLAGS_None = 0,
			FORMFLAGS_VScroll = 1,
			FORMFLAGS_HScroll = 2,
			FORMFLAGS_AllowResize = 4
			};

		DBFORM (
			);

		DBFORM (
			const RVC::DBFORM& rhs
			);

		virtual ~DBFORM (
			);

		RVC::DBFORM& operator= (
			const RVC::DBFORM& rhs
			);

		//! Obtain form flags
		//! @return Flags for the form structure
		FORMFLAGS GetFlags (
			) const;

		//! Get reference to FORMFIELD list
		//! @return Constant reference to form field list
		const SIMPLE_ARRAY<FORMFIELD>& GetFormFieldList (
			) const;

		//! Obtain form window height
		//! @return form window height
		INT16 GetHeight (
			) const;

		//! Obtain form window partial height
		//! @return form window partial height
		INT16 GetPartialHeight (
			) const;

		//! Obtain form window partial width
		//! @return form window partial width
		INT16 GetPartialWidth (
			) const;

		//! Obtain database table name this form is associated with
		//! @return form table name
		const RVC::OBJECTNAME& GetTableName (
			) const;

		//! Obtain form window width
		//! @return form window width
		INT16 GetWidth (
			) const;

		//! Obtain form window x position (uppper left)
		//! @return form window x position
		INT16 GetWindowX (
			) const;

		//! Obtain form window y position (uppper left)
		//! @return form window y position
		INT16 GetWindowY (
			) const;

		//! Set form flags
		void SetFlags (
			FORMFLAGS flags
			);

		//! Set form field list
		void SetFormFieldList (
			const SIMPLE_ARRAY<FORMFIELD>& FormFieldList
			);

		//! Set form window height
		void SetHeight (
			INT16 height
			);

		//! Set form window partial height
		void SetPartialHeight (
			INT16 PartialHeight
			);

		//! Set form window partial width
		void SetPartialWidth (
			INT16 PartialWidth
			);

		//! Set database table name to associate DBForm with
		void SetTableName (
			const RVC::OBJECTNAME& TableName
			);

		//! Set form window width
		void SetWidth (
			INT16 height
			);

		//! Set form window x position (upper left)
		void SetWindowX (
			INT16 x
			);

		//! Set form window y position (upper left)
		void SetWindowY (
			INT16 x
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		SIMPLE_ARRAY<FORMFIELD> m_FormField;
		OBJECTNAME m_TableName;
		FORMFLAGS m_Flags;
		INT16 m_Height;
		INT16 m_PartialHeight;
		INT16 m_Width;
		INT16 m_PartialWidth;
		INT16 m_X;
		INT16 m_Y;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


//!	Structures for Forms.
struct DBFORM::FORMFIELD {

	//! Control type, values for FORMFIELD::type member.
	enum CTRLTYPE {
		CTRLTYPE_Default =		1,			//!< Default, based on field type and usage
		CTRLTYPE_Label =			2			//!< Show label only
		};

	//! Values for FORMFIELD::flags member.
	enum FLAGS {
		FLAG_CheckMin =			0x0001,	//!< Check minimum value
		FLAG_CheckMax =			0x0002,	//!< Check maximum value
		FLAG_AlignRight =			0x0004,	//!< Right align value in edit control
		};

	RVC::OBJECTNAME fname;					//!< Field name
	FIXEDSTRING<80> desc; 					//!< Label to put on the field
	DOUBLE min, max;							//!< For range checking
	DOUBLE dft, sparedouble;
	INT16 lin, partial_lin;					//!< Position in chars and 100ths of a char
	INT16 col, partial_col;

	INT16 end_lin, end_partial_lin;		//!< Lower right corner
	INT16 end_col, end_partial_col;		//!< For "frames"

	INT16 lins, cols;							//!< Shape of editable portion
	INT16 partial_lins, partial_cols;	//!< Partial lines / cols

	INT16 places; 								//!< Number of decimal places to show
	INT16 type;									//!< FORMFIELD::CTRLTYPE

	UINT32 flags;								//!< FORMFIELD::FLAGS
	INT32 parent;								//!< For fields in scrolled list
	UINT8 spare[4];
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DBFORM::FORMFIELD::FLAGS)
#endif


}		// End of RVC namespace

#endif		// INC_RVC_DBFORM_H
