/**
 * \file rvc/hyperhistogram.h 
 * \brief interface for the RVC::HYPERHISTOGRAM class.
 *
 * \if NODOC
 * $Id: hyperhistogram.h_v 1.4 2011/12/09 23:01:34 mju Exp $
 *
 * $Log: hyperhistogram.h_v $
 * Revision 1.4  2011/12/09 23:01:34  mju
 * Remove redundant hdr.
 *
 * Revision 1.1  2006/03/03 22:39:36  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_HYPERHISTOGRAM_H
#define INC_RVC_HYPERHISTOGRAM_H

#ifndef	INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef	INC_MI32_RANGE_H
	#include <mi32/range.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class STAT_UNIVARIATE;
namespace RVC {
	struct HYPERHISTBANDINFO;
	class HYPERSPECTRAL;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_HyperspectralHistogram objects. 
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_HyperspectralHistogram.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Hyperspectral
class HYPERHISTOGRAM : public RVC::OBJECT {
	public:
	
		HYPERHISTOGRAM ();

		HYPERHISTOGRAM (
			const RVC::HYPERHISTOGRAM& rhs
			);

		virtual ~HYPERHISTOGRAM ();

		HYPERHISTOGRAM& operator= (
			const HYPERHISTOGRAM& rhs
			);

		//! Get histogram bin index corresponding to specified cell value.
		//! @return Index, may not correspond to valid bin.
		int GetBinIndexFromValue (
			double CellValue
			) const;

		//! Get histogram bin interval.
		//! @return Bin interval
		double GetBinInterval (
			) const;

		//! Get maximum histogram bin value.
		//! This returns the highest cell value which could be counted in last histogram bin.
		//! @return Maximum value, may be less than maximum cell value.
		double GetBinMaxValue (
			) const;

		//! Get minimum histogram bin value.
		//! This returns the lowest cell value which could be counted in first histogram bin.
		//! @return Minimum value, may be greater than minimum cell value.
		double GetBinMinValue (
			) const;

		//! Get range of histogram bin values.
		const DOUBLE_RANGE& GetBinRange (
			) const { return (m_BinRange); }

		//! Get cell count depending on cell value
		//! @return Cell count of cell value
		UINT32 GetCellCount (
			double CellValue
			) const;

		//! Get array of histogram cell counts
		//! @return Cell count list
		const SIMPLE_ARRAY<UINT32>& GetCellCounts (
			) const;

		//! Get maximum cell value.
		//! @return Cell value, may be less than minimum bin value.
		double GetCellMaxValue (
			) const { return (m_CellRange.GetMaximum()); }

		//! Get minimum cell value.
		double GetCellMinValue (
			) const { return (m_CellRange.GetMinimum()); }

		//! Get cell value range.
		void GetCellRange (
			DOUBLE_RANGE& CellRange
			) const { CellRange = m_CellRange; }

		//! Get column sampling interval
		//! @return Column sampling interval
		UINT16 GetColumnSampleInterval (
			) const;
			
		//! Get hyperspectral band that this histogram was generated for
		//! @return Band index
		INT32 GetBand (
			) const;

		//! Get number of cells with values greater than maximum bin value.
		//! @return Cell count.
		UINT32 GetGreaterThanCount (
			) const;
			
		//! Get number of cells with values less than minimum bin value.
		//! @return Cell count.
		UINT32 GetLessThanCount (
			) const;
			
		//! Get line sampling interval
		//! @return Line sampling interval
		UINT16 GetLineSampleInterval (
			) const;
			
		//! Get the computed mean histogram value
		//! @return Mean value
		double GetMean (
			) const;

		//! Get number of null cells in the raster
		//! @return Number of raster null cells
		UINT32 GetNullCellCount (
			) const;

		//! Get statistics as STAT_UNIVARIATE.
		ERRVALUE GetStatistics (
			STAT_UNIVARIATE& stats				//!< Statistics returned
			) const;
			
		//! Get the computed standard deviation histogram value
		//! @return Standard Deviation value
		double GetStdDev (
			) const;

		//! Create a histogram.
		ERRVALUE Make (
			const HYPERSPECTRAL& parent,
			INT32 BandNum,
#ifdef RVC_CREDENTIALS_MANDATORY
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read a histogram under the hyperspectral object for the given band.  
		//! If the existing histogram is old or does not exist, a histogram is created.
		//! If a different band is given that from the previous Read(), the previous band 
		//! information is replaced.
		ERRVALUE Read (
			const HYPERSPECTRAL& parent,
			INT32 BandNum,
#ifdef RVC_CREDENTIALS_MANDATORY
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Set bin range.
		void SetBinRange (
			const DOUBLE_RANGE& range
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SIMPLE_ARRAY<UINT32> m_CellCount;		// Array of cell counts
		DOUBLE_RANGE m_BinRange;					// Overall bin range
		double m_BinInterval;						// Bin interval
		double m_Mean;	   							// Mean value within selected bin range
		double m_MeanFull;							//!<  Mean value for entire data range 
		double m_StdDev;   							// Standard Deviation within selected bin range
		double m_Sum;									//!<  Sum of values 
		double m_Sum2;									//!<  Sum of squares of values 
		UINT32 m_CountLtMin;							// Number of cells with values less minimum bin
		UINT32 m_CountGtMax;							// Number of cells with values greater maximum bin
		UINT32 m_CountNull;							// Number of NULL cells in the raster
		DOUBLE_RANGE m_CellRange;					// Overall cell range
		INT32 m_BandNum;

		void SetInfo (const HYPERHISTBANDINFO& histinfo);
			
		// Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		// End of RVC namespace

#endif	// INC_RVC_HYPERHISTOGRAM_H
