/**
 * \file rvc/opsurfacefitting.h
 * \brief Definitions for RVC::OP::SURFACEFITTING class.
 *
 * \if NODOC
 * $Id: opsurfacefitting.h_v 1.16 2010/02/03 23:11:55 mju Exp $
 *
 * $Log: opsurfacefitting.h_v $
 * Revision 1.16  2010/02/03 23:11:55  mju
 * Move blanking distance to method_parameters.
 *
 * Revision 1.15  2010/02/03 22:42:15  mju
 * Add blanking support.
 *
 * Revision 1.14  2009/06/26 12:50:49  mju
 * Use component class.
 *
 * Revision 1.13  2009/06/25 17:47:43  mju
 * Change processTriangleInterpolation to take ptr to srcParms like other methods.
 *
 * Revision 1.12  2008/09/02 21:40:13  mju
 * Fix doxygen warning.
 *
 * Revision 1.11  2008/06/24 17:52:18  mju
 * Add duplicate handling for mincurve.
 *
 * Revision 1.10  2008/06/24 17:02:40  mju
 * Use needed hdrs so can include first.
 *
 * Revision 1.9  2008/06/17 18:03:50  vdronov
 * added shape source
 *
 * Revision 1.8  2007/08/20 14:33:02  mju
 * Remove illegal qualifiers.
 *
 * Revision 1.7  2007/05/21 19:43:31  vdronov
 * added modified quintic
 *
 * Revision 1.6  2007/03/08 23:50:07  vdronov
 * added kriging
 *
 * Revision 1.5  2007/02/19 21:59:57  vdronov
 * added unicurve
 *
 * Revision 1.4  2007/02/16 23:24:27  vdronov
 * added bidirectional
 *
 * Revision 1.3  2007/02/12 16:29:31  vdronov
 * added minimum curvature
 *
 * Revision 1.1  2007/01/11 22:59:43  vdronov
 * Initial revision
 *
 * \endif
**/

#ifndef  INC_RVC_OPSURFACEFITTING_H
#define  INC_RVC_OPSURFACEFITTING_H

#ifndef	INC_RVC_OPCOMPONENT_H
	#include <rvc/opcomponent.h>
#endif

#include <float.h>

#ifndef	INC_MI32_KRIGING_H
#include <mi32/kriging.h>
#endif

#ifndef	INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

#ifndef	INC_MI32_POLYLINE_H
#include <mi32/polyline.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif


namespace RVC {
namespace OP {

//! Class for surface fitting.
class SURFACEFITTING {
	public:

	   //! enum for available sources for surface fitting.
		enum SOURCE {
			SOURCE_Invalid = -1,
			SOURCE_First = 0,
			SOURCE_Vector = 0,
			SOURCE_TIN = 1,
			SOURCE_Table = 2,
			SOURCE_Shape = 3,
			SOURCE_Last = 3,
			SOURCE_Count = 4
			};

	   //! enum for available methods of surface fitting
		enum METHOD {
			METHOD_Invalid = -1,
			METHOD_First = 0,
			METHOD_MinimumCurvature = 0,
			METHOD_UnivariateCurve = 1,
			METHOD_InverseDistance = 2,
			METHOD_Profiles = 3,
			METHOD_Polynomial = 4,
			METHOD_Kriging = 5,
			METHOD_TriangleInterpolation = 6,
			METHOD_Bidirectional = 7,
			METHOD_Last = 7,
			METHOD_Count = 8
			};

		//! Class describes parameters of source object.
		class SOURCE_PARAMETERS {
			public:

				SOURCE_PARAMETERS (
					const SOURCE source
					);

				virtual ~SOURCE_PARAMETERS (
					);

				SOURCE GetSource (
					) const { return m_Source; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				SOURCE_PARAMETERS (
					);

				const SOURCE m_Source;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Class describes parameters of Vector source object.
		class VECTOR_PARAMETERS : public SOURCE_PARAMETERS {
			public:

				//! enum describes available types of source components.
				enum TYPE {
					TYPE_Point = 0,
					TYPE_Line = 1
					};

				VECTOR_PARAMETERS (
					);

				~VECTOR_PARAMETERS (
					);

				VECTOR_PARAMETERS& operator= (
					const VECTOR_PARAMETERS& 
					);

				bool GetComponent (
					const TYPE type,
					COMPONENT& component
					) const;

				const OP::COMPONENT& GetComponentLine (
					) const { return (m_Line); }

				const OP::COMPONENT& GetComponentPoint (
					) const { return (m_Point); }

				void SetComponent (
					const TYPE type,
					const OP::COMPONENT& component
					);

				void SetComponentLine (
					const OP::COMPONENT& component
					) { m_Line = component; }

				void SetComponentPoint (
					const OP::COMPONENT& component
					) { m_Point = component; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				COMPONENT m_Point;
				COMPONENT m_Line;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Class describes parameters of Vector source object.
		class TIN_PARAMETERS : public SOURCE_PARAMETERS {
			public:

				//! enum describes available types of source components.
				enum TYPE {
					TYPE_Node = 0,
					TYPE_Edge = 1,
					TYPE_Triangle = 2
					};

				TIN_PARAMETERS (
					);

				~TIN_PARAMETERS (
					);

				TIN_PARAMETERS& operator= (
					const TIN_PARAMETERS& 
					);

				bool GetComponent (
					const TYPE type,
					COMPONENT& component
					) const;

				const OP::COMPONENT& GetComponentEdge (
					) const { return (m_Edge); }

				const OP::COMPONENT& GetComponentNode (
					) const { return (m_Node); }

				const OP::COMPONENT& GetComponentTriangle (
					) const { return (m_Triangle); }

				void SetComponent (
					const TYPE type,
					const COMPONENT& component
					);

				void SetComponentEdge (
					const OP::COMPONENT& component
					) { m_Edge = component; }

				void SetComponentNode (
					const OP::COMPONENT& component
					) { m_Node = component; }

				void SetComponentTriangle (
					const OP::COMPONENT& component
					) { m_Triangle = component; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				COMPONENT m_Node;
				COMPONENT m_Edge;
				COMPONENT m_Triangle;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Class describes parameters of table source object constructing 3D points from each record in table 
		class TABLE_PARAMETERS : public SOURCE_PARAMETERS {
			public:

				//! enum describes id's of field names constructing 3D points.
				enum NAME {
					NAME_XField = 0,
					NAME_YField = 1,
					NAME_ZField = 2,
					NAME_Count = 3
					};

				TABLE_PARAMETERS (
					);

				~TABLE_PARAMETERS (
					);

				TABLE_PARAMETERS& operator= (
					const TABLE_PARAMETERS& 
					);

			//! Returns COORDREFSYS for constructed 3D points.
				const SPATREF::COORDREFSYS& GetCoordRefSys (
					) const { return (m_CoordRefSys); }

				const COMPONENT& GetComponent (
					) const { return (m_Pin); }

				//! Returns field name for each coordinate of constructed 3D point.
				void GetName (
					const NAME id,
					RVC::OBJECTNAME& name
					) const;

				//! Returns generic query for constructing 3D point.
				const MISTRING& GetQuery (
					) const { return (m_Query); }

				//! Returns unit code for x coordinate of constructed 3D point.
				UINT8 GetXUnitCode (
					) const { return (m_XUnitCode); }

				//! Returns unit code for y coordinate of constructed 3D point.
				UINT8 GetYUnitCode (
					) const { return (m_YUnitCode); }

				//! Returns unit code for z coordinate of constructed 3D point.
				UINT8 GetZUnitCode (
					) const { return (m_ZUnitCode); }

				//! Set COORDREFSYS for constructed 3D points.
				void SetCoordRefSys (
					const SPATREF::COORDREFSYS& crs
					) { m_CoordRefSys = crs; }

				void SetComponent (
					const COMPONENT& component
					) { m_Pin = component; }

				//! Set field name for each coordinate of constructed 3D point.
				void SetName (
					const NAME id,
					const RVC::OBJECTNAME& name
					);

				//! Set generic query for constructing 3D point.
				void SetQuery (
					const MISTRING& query
					) { m_Query = query; }

				//! Set unit code for x coordinate of constructed 3D point.
				void SetXUnitCode (
					const UINT8 unitcode
					) { m_XUnitCode = unitcode; }

				//! Set unit code for y coordinate of constructed 3D point.
				void SetYUnitCode (
					const UINT8 unitcode
					) { m_YUnitCode = unitcode; }

				//! Set unit code for z coordinate of constructed 3D point.
				void SetZUnitCode (
					const UINT8 unitcode
					) { m_ZUnitCode = unitcode; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				COMPONENT m_Pin;
				RVC::OBJECTNAME m_Names[NAME_Count];
				SPATREF::COORDREFSYS m_CoordRefSys;
				MISTRING m_Query;
				UINT8 m_XUnitCode;
				UINT8 m_YUnitCode;
				UINT8 m_ZUnitCode;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Class describes parameters of Shape source object.
		class SHAPE_PARAMETERS : public SOURCE_PARAMETERS {
			public:

				SHAPE_PARAMETERS (
					);

				~SHAPE_PARAMETERS (
					);

				SHAPE_PARAMETERS& operator= (
					const SHAPE_PARAMETERS& 
					);

				const COMPONENT& GetComponent (
					) const { return (m_Component); }

				void SetComponent (
					const COMPONENT& component
					) { m_Component = component; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				COMPONENT m_Component;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Class describes parameters of destination object Raster.
		class DESTINATION_PARAMETERS {
			public:

				DESTINATION_PARAMETERS (
					);

				~DESTINATION_PARAMETERS (
					);

				RVC::IMAGE::CELLTYPE GetCellType (
					) const { return m_CellType; }

				UINT16 GetCompressionQualityRatio (
					) const { return m_CompressionQualityRatio; }

				RVC::RASTER::COMPTYPE GetCompressionType (
					) const { return m_CompressionType; }

				INT32 GetNumColumns (
					) const { return m_NumColumns; }

				INT32 GetNumLines (
					) const { return m_NumLines; }

				//! Returns transformation from object to Map of destination Raster.
				const TRANS2D_MAPGEN& GetTransformation (
					) const { return m_Transformation; }

				void SetCellType (
					const RVC::IMAGE::CELLTYPE CellType
					) { m_CellType = CellType; }

				void SetCompressionQualityRatio (
					const UINT16 CompressionQualityRatio
					) { m_CompressionQualityRatio = CompressionQualityRatio; }

				void SetCompressionType (
					const RVC::RASTER::COMPTYPE CompressionType
					); 
				void SetNumColumns (
					const INT32 NumColumns
					) { m_NumColumns = NumColumns; }

				void SetNumLines (
					const INT32 NumLines
					) { m_NumLines = NumLines; }

				//! Set transformation from object to Map of destination Raster.
				void SetTransformation (
					const TRANS2D_MAPGEN& transformation
					);

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				INT32 m_NumLines;
				INT32 m_NumColumns;
				RVC::IMAGE::CELLTYPE m_CellType;
				TRANS2D_MAPGEN m_Transformation;
				RVC::RASTER::COMPTYPE m_CompressionType;
				UINT16 m_CompressionQualityRatio;

				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Base class for all surface fitting.
		class METHOD_PARAMETERS {
			public:

				METHOD_PARAMETERS (
					const METHOD method
					);

				virtual ~METHOD_PARAMETERS (
					);

				//! Get blanking distance.
				UINT16 GetBlankDistance (
					) const { return (m_BlankDistance); }

				//! Return enum of method in use
				METHOD GetMethod (
					) const { return m_Method; }

				//! Set blanking distance.
				void SetBlankDistance (
					UINT16 BlankDistance						//!< Blanking distance in cells, 0 for no blanking
					) { m_BlankDistance = BlankDistance; }

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT

				const METHOD m_Method;
				UINT16 m_BlankDistance;

				METHOD_PARAMETERS (
					); // not implemented

				static METHOD Check (
					const METHOD method
					);

				#endif // GENERATING_DOXYGEN_OUTPUT

			};

		//! Class for surface fitting method from .
		class MINIMUMCURVATURE_PARAMETERS : public METHOD_PARAMETERS {
			public:

				enum CURVATURE {
					CURVATURE_Linear = 0,
					CURVATURE_Logarithmic = 1
					};

				enum DUPLICATE {
					DUPLICATE_First = 0,
					DUPLICATE_Minimum,
					DUPLICATE_Maximum,
					DUPLICATE_Last
					};

				enum INITIALIZATION {
					INITIALIZATION_InverseDistance = 0,
					INITIALIZATION_Profiles = 1
					};

				enum GRIDRATIO {
					GRIDRATIO_1 = 1,
					GRIDRATIO_2 = 2,
					GRIDRATIO_4 = 4,
					GRIDRATIO_8 = 8,
					GRIDRATIO_16 = 16
					};

				enum SEARCHAREA {
					SEARCHAREA_Circle = 0,
					SEARCHAREA_Square = 1,
					SEARCHAREA_Diamond = 2
					};

				MINIMUMCURVATURE_PARAMETERS (
					); 

				double GetAllowedVariation (
					) const { return m_AllowedVariation; }

				CURVATURE GetCurvature (
					) const { return m_Curvature; }

				DUPLICATE GetDuplicateHandling (
					) const { return (m_DuplicateHandling); }

				GRIDRATIO GetGridRatio ( 
					) const { return m_GridRatio; }

				INITIALIZATION GetInitialization (
					) const { return m_Initialization; }

				int GetDistanceUnit (
					) const { return m_DistanceUnit; }

				double GetMatchingTolerance (
					) const { return m_MatchingTolerance; }

				INT32 GetMaximumIterations (
					) const { return m_MaximumIterations; }

				double GetMinimumMatches (
					) const { return m_MinimumMatches; }

				SEARCHAREA GetSearchArea (
					) const { return m_SearchArea; }

				double GetSearchDistance (
					) const { return m_SearchDistance; }

				double GetTension (
					) const { return m_Tension; }

				double GetWeightingPower (
					) const { return m_WeightingPower; }

				double SetAllowedVariation (
					const double variation
					) { m_AllowedVariation = MAX(0.0, variation); return m_AllowedVariation; }

				void SetCurvature (
					const CURVATURE curvature
					) { m_Curvature = curvature; }

				void SetDuplicateHandling (
					DUPLICATE DuplicateHandling
					) { m_DuplicateHandling = DuplicateHandling; }

				void SetGridRatio (
					const GRIDRATIO gridratio
					) { m_GridRatio = gridratio; }

				void SetInitialization (
					const INITIALIZATION initialization
					) { m_Initialization = initialization; }

				void SetDistanceUnit (
					const int unit 
					) { m_DistanceUnit = unit; }

				double SetMatchingTolerance (
					const double tolerance
					) { m_MatchingTolerance = MAX(tolerance, 0.0); return m_MatchingTolerance; }

				INT32 SetMaximumIterations (
					const INT32 iterations
					) { m_MaximumIterations = bound(iterations, 1, 1000); return m_MaximumIterations; }

				double SetMinimumMatches (
					const double matches
					) { m_MinimumMatches = bound(matches,10.0, 100.0); return m_MinimumMatches; }

				void SetSearchArea (
					const SEARCHAREA area 
					) { m_SearchArea = area; }

				double SetSearchDistance (
					const double distance 
					) { m_SearchDistance = bound(distance, DBL_MIN, DBL_MAX); return m_SearchDistance; }

				double SetTension (
					const double tension 
					) { m_Tension = bound(tension, 0.0, 1.0); return m_Tension; }

				double SetWeightingPower (
					const double power 
					) { m_WeightingPower = bound(power, 1.0, 20.0); return m_WeightingPower; }

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				CURVATURE m_Curvature;
				GRIDRATIO m_GridRatio;
				INITIALIZATION m_Initialization;
				DUPLICATE m_DuplicateHandling;
				SEARCHAREA m_SearchArea;
				INT32 m_MaximumIterations;
				int m_DistanceUnit;
				double m_AllowedVariation;
				double m_MatchingTolerance;
				double m_MinimumMatches;
				double m_SearchDistance;
				double m_Tension;
				double m_WeightingPower;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Class for surface fitting method from .
		class UNIVARIATECURVE_PARAMETERS : public METHOD_PARAMETERS {
			public:

				enum INTERPOLATION {
					INTERPOLATION_Linear = 0,
					INTERPOLATION_Cubic = 1,
					INTERPOLATION_Hermite = 2
					};

				enum SLOPE {
					SLOPE_Linear = 0,
					SLOPE_Quadratic = 1
					};

				UNIVARIATECURVE_PARAMETERS (
					); 

				SLOPE GetSlope (
					) const { return m_Slope; }

				INTERPOLATION GetInterpolation (
					) const { return m_Interpolation; }

				void SetSlope (
					const SLOPE slope
					) { m_Slope = slope; }

				void SetInterpolation (
					const INTERPOLATION interpolation
					) {m_Interpolation = interpolation; }

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				INTERPOLATION m_Interpolation;
				SLOPE m_Slope;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Class for surface fitting method from .
		class INVERSEDISTANCE_PARAMETERS : public METHOD_PARAMETERS {
			public:

				enum SEARCHAREA {
					SEARCHAREA_Circle = 0,
					SEARCHAREA_Square = 1,
					SEARCHAREA_Diamond = 2
					};

				INVERSEDISTANCE_PARAMETERS (
					); 

				int GetDistanceUnit (
					) const { return m_DistanceUnit; }

				SEARCHAREA GetSearchArea (
					) const { return m_SearchArea; }

				double GetSearchDistance (
					) const { return m_SearchDistance; }

				double GetWeightingPower (
					) const { return m_WeightingPower; }

				void SetDistanceUnit (
					const int unit 
					) { m_DistanceUnit = unit; }

				void SetSearchArea (
					const SEARCHAREA area 
					) { m_SearchArea = area; }

				double SetSearchDistance (
					const double distance 
					) { m_SearchDistance = bound(distance, DBL_MIN, DBL_MAX); return m_SearchDistance; }

				double SetWeightingPower (
					const double power 
					) { m_WeightingPower = bound(power, 1.0, 20.0); return m_WeightingPower; }

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				double m_SearchDistance;
				int m_DistanceUnit;
				SEARCHAREA m_SearchArea;
				double m_WeightingPower;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Class for surface fitting method from .
		class PROFILES_PARAMETERS : public METHOD_PARAMETERS {
			public:

				PROFILES_PARAMETERS (
					); 

				int GetDistanceUnit (
					) const { return m_DistanceUnit; }

				double GetSearchDistance (
					) const { return m_SearchDistance; }

				void SetDistanceUnit (
					const int unit 
					) { m_DistanceUnit = unit; }

				double SetSearchDistance (
					const double distance 
					) { m_SearchDistance = bound(distance, DBL_MIN, DBL_MAX); return m_SearchDistance; }


				private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				double m_SearchDistance;
				int m_DistanceUnit;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Class for surface fitting method from .
		class POLYNOMIAL_PARAMETERS : public METHOD_PARAMETERS {
			public:

				POLYNOMIAL_PARAMETERS (
					); 

				UINT8 GetOrder (
					) const { return m_Order; }

				bool GetSaveCoefficients (
					) const { return m_SaveCoefficients; }

				UINT8 SetOrder (
					const UINT8 order
					) { m_Order = bound(order, 1, 20); return m_Order; }

				void SetSaveCoefficients (
					const bool save
					) { m_SaveCoefficients = save; }

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				UINT8 m_Order;
				bool m_SaveCoefficients;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Class for surface fitting method from .
		class KRIGING_PARAMETERS : public METHOD_PARAMETERS {
			public:


				KRIGING_PARAMETERS (
					); 

				const KRIGING& GetKriging (
					) const;

				KRIGING& GetKriging (
					);

				bool GetMakeDrift (
					) const { return m_MakeStandardError; }

				bool GetMakeStandardError (
					) const { return m_MakeDrift; }

				void SetMakeDrift (
					const bool make
					) { m_MakeStandardError = make; }

				void SetMakeStandardError (
					const bool make
					) { m_MakeDrift = make; }


			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				KRIGING m_Kriging;
				bool m_MakeStandardError;
				bool m_MakeDrift;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Class for surface fitting method from .
		class TRIANGLEINTERPOLATION_PARAMETERS : public METHOD_PARAMETERS {
			public:

				enum INTERPOLATION {
					INTERPOLATION_Linear = 0,
					INTERPOLATION_Quintic = 1,
					INTERPOLATION_ModifiedQuintic = 2,
					INTERPOLATION_Nonic = 3
					};

				TRIANGLEINTERPOLATION_PARAMETERS (
					); 

				INTERPOLATION GetInterpolation (
					) const { return m_Interpolation; }

				void SetInterpolation (
					const INTERPOLATION interpolation
					) { m_Interpolation = interpolation; }

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				INTERPOLATION m_Interpolation;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Class for surface fitting method from .
		class BIDIRECTIONAL_PARAMETERS : public METHOD_PARAMETERS {
			public:

				enum DIRECTIONTYPE {
					DIRECTIONTYPE_Manual = 0,
					DIRECTIONTYPE_Auto = 1
					};

				BIDIRECTIONAL_PARAMETERS (
					); 

				POLYLINE::SPLINE GetAcrossLineSplineType (
					) const { return m_AcrossLineSplineType; }

				POLYLINE::SPLINE GetAlongLineSplineType (
					) const { return m_AlongLineSplineType; }

				double GetAzimuth (
					) const { return m_Azimuth; }

				DIRECTIONTYPE GetDirectionType (
					) const { return m_DirectionType; }

				void SetAcrossLineSplineType (
					const POLYLINE::SPLINE spline
					) { m_AcrossLineSplineType = spline; }

				void SetAlongLineSplineType (
					const POLYLINE::SPLINE spline
					) { m_AlongLineSplineType = spline; }

				void SetAzimuth (
					const double azimuth
					) { m_Azimuth = azimuth; }

				void SetDirectionType (
					const DIRECTIONTYPE type
					) { m_DirectionType = type; }

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				POLYLINE::SPLINE m_AlongLineSplineType;
				POLYLINE::SPLINE m_AcrossLineSplineType;
				DIRECTIONTYPE m_DirectionType;
				double m_Azimuth;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Base class for geometric element iterator.
		class GEOMETRIC_ITERATOR {
			public:

				//! Default ctor	
				//! Copy ctor and assignment operators are private
				GEOMETRIC_ITERATOR (
					) {
					}

				//! Default dtor
				virtual ~GEOMETRIC_ITERATOR (
					) {}

				//! Pre-increment operator.
				GEOMETRIC_ITERATOR& operator++ (
					) { return (v_Increment()); }
					
				//! Get element ID for the iterator
				INT64 GetElementID (
					) const { return (v_GetElementID()); }

				//! Get current position (0-1) or -1 if cannot determine
				double GetPosition (
					) const { return (v_GetPosition()); }

				//! Determine if done with iteration.
				bool IsDone (
					) const { return (v_IsDone()); }

				//! Restart to beginning (if supported);
				//! @return TRUE if successful, FALSE if not supported, < 0 if error
				int Restart (
					) { return (v_Restart()); }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				//! Get current Element ID.
				//! Default implementation returns -1.
				virtual INT64 v_GetElementID (
					) const { return -1; }
					
				//! Get current position.
				//! Default implementation returns 1.0
				virtual double v_GetPosition (
					) const { return 1.0; }

				//! Pre-increment.
				virtual GEOMETRIC_ITERATOR& v_Increment () = 0;

				//! Determine if done.
				virtual bool v_IsDone () const = 0;

				//! Restart to beginning.
				//! @return TRUE if successful, FALSE if not supported, < 0 if error
				//! Default returns FALSE.
				virtual ERRVALUE v_Restart (
					) { return false; }

				GEOMETRIC_ITERATOR (const GEOMETRIC_ITERATOR& rhs);
				GEOMETRIC_ITERATOR& operator=(const GEOMETRIC_ITERATOR& rhs);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Point iterator.
		class GEOMETRIC_ITERATOR_POINT : public GEOMETRIC_ITERATOR {
			public:

				//! Default ctor	
				explicit GEOMETRIC_ITERATOR_POINT (
					DIMENSION dim = DIMENSION_2D
					) : m_Dimension(dim) {}

				//! Default dtor
				virtual ~GEOMETRIC_ITERATOR_POINT (
					) {}

				GEOMETRIC_ITERATOR_POINT& operator=(
					const GEOMETRIC_ITERATOR_POINT& rhs
					) { if (this != &rhs) { m_Dimension = rhs.m_Dimension; } return (*this); }

				//! Dereference operator.
				const DPOINT3D& operator* (
					) const { return (m_Point); }

				//! Dereference operator.
				const DPOINT3D* operator-> (
					) const { return (&m_Point); }

			protected:
				DPOINT3D m_Point;

				DIMENSION GetDimension (
					) const { return (m_Dimension); }
				
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DIMENSION m_Dimension;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Line iterator.
		class GEOMETRIC_ITERATOR_LINE : public GEOMETRIC_ITERATOR {
			public:

				//! Default ctor	
				explicit GEOMETRIC_ITERATOR_LINE (
					DIMENSION dim = DIMENSION_2D
					) : m_Dimension(dim), m_Polyline(false, dim) {}

				//! Default dtor
				virtual ~GEOMETRIC_ITERATOR_LINE (
					) {}

				GEOMETRIC_ITERATOR_LINE& operator=(
					const GEOMETRIC_ITERATOR_LINE& rhs
					) { if (this != &rhs) { m_Dimension = rhs.m_Dimension; } return (*this); }

				//! Dereference operator.
				const POLYLINE& operator* (
					) const { return (m_Polyline); }

				//! Dereference operator.
				const POLYLINE* operator-> (
					) const { return (&m_Polyline); }

			protected:
				POLYLINE m_Polyline;

				DIMENSION GetDimension (
					) const { return (m_Dimension); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DIMENSION m_Dimension;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		class GEOMETRIC_SOURCE {
			public:

				GEOMETRIC_SOURCE (
					) {}

				virtual ~GEOMETRIC_SOURCE (
					) {}

				enum COMPONENT {
					COMPONENT_Point = 0,
					COMPONENT_Line = 1,
					COMPONENT_SubRegion = 2
					};
		 
				bool IsComponentAvailable (
					const COMPONENT component
					) const { return v_IsComponentAvailable(component); }
		 
			   GEOMETRIC_ITERATOR_POINT* GetPoints (
					) { return v_GetPoints(); }

				GEOMETRIC_ITERATOR_LINE* GetLines (
					) { return v_GetLines(); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				virtual bool v_IsComponentAvailable (const COMPONENT component) const = 0;
			   virtual GEOMETRIC_ITERATOR_POINT* v_GetPoints() = 0;
				virtual GEOMETRIC_ITERATOR_LINE* v_GetLines() = 0;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Perform blanking (set to null) of cells far from source elements.
		static ERRVALUE DoBlanking (
		   const RVC::OBJITEM& ObjItemSrc,
			const RVC::OBJITEM& ObjItemTgt,
			const SOURCE_PARAMETERS& SrcParameters,
			const DESTINATION_PARAMETERS& DestParameters,
			UINT16 BlankDistance
			);

		//! Surface fitting
		static ERRVALUE Process (
		   const RVC::OBJITEM& SrcObjItem,
	      RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const METHOD_PARAMETERS* MethodParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

		//! Minimum curvature surface fitting
		static ERRVALUE ProcessMinimumCurvature (
			const RVC::OBJITEM& SrcObjItem,
			RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const MINIMUMCURVATURE_PARAMETERS& MinimumCurvatureParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			); 

		//! Univariate curve surface fitting
		static ERRVALUE ProcessUnivariateCurve (
			const RVC::OBJITEM& SrcObjItem,
			RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const UNIVARIATECURVE_PARAMETERS& UnivariateCurveParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			); 

		//! Inverse distance surface fitting
		static ERRVALUE ProcessInverseDistance (
			const RVC::OBJITEM& SrcObjItem,
			RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const INVERSEDISTANCE_PARAMETERS& InverseDistanceParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			); 

		//! Profiles surface fitting
		static ERRVALUE ProcessProfiles (
			const RVC::OBJITEM& SrcObjItem,
			RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const PROFILES_PARAMETERS& ProfilesParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			); 

		//! Polynomial surface fitting
		static ERRVALUE ProcessPolynomial (
		   const RVC::OBJITEM& SrcObjItem,
	      RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const POLYNOMIAL_PARAMETERS& PolynomialParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

		//! Kriging surface fitting
		static ERRVALUE ProcessKriging (
			const RVC::OBJITEM& SrcObjItem,
			RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const KRIGING_PARAMETERS& KrigingParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			); 

		//! Triangle interpolation surface fitting
		static ERRVALUE ProcessTriangleInterpolation (
			const RVC::OBJITEM& SrcObjItem,
			RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const TRIANGLEINTERPOLATION_PARAMETERS& TriangleInterpolationParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

		//! Bidirectional surface fitting
		static ERRVALUE ProcessBidirectional (
			const RVC::OBJITEM& SrcObjItem,
			RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const BIDIRECTIONAL_PARAMETERS& BidirectionalParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

		//! Get points as polyline from any source
		static ERRVALUE GetPoints (
			const RVC::OBJITEM& SrcObjItem, 
			const SOURCE_PARAMETERS* SrcParameters, 
			POLYLINE& polyline
			);

		//! Get geometric source
		static RVC::OP::SURFACEFITTING::GEOMETRIC_SOURCE* GetGeometricSource (
			const RVC::OBJITEM& SrcObjItem, 
			const SOURCE_PARAMETERS* SrcParameters 
			);

		//! Make destination raster
		static ERRVALUE MakeDestination (
			RVC::OBJITEM& DestObjItem,
			const RVC::OP::SURFACEFITTING::DESTINATION_PARAMETERS& DestParameters, 
			RVC::RASTER& raster
			);

		//! Get transformation from source object coordinates to map coordinates
		static ERRVALUE GetTransformation (
			const RVC::OBJITEM& SrcObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			TRANS2D_MAPGEN& transformation
			);

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		static ERRVALUE GetVectorPoints (const RVC::OBJITEM& SrcObjItem, const VECTOR_PARAMETERS& VectorParameters, POLYLINE& polyline);
		static ERRVALUE GetTablePoints (const RVC::OBJITEM& SrcObjItem, const TABLE_PARAMETERS& TableParameters, POLYLINE& polyline);
		static ERRVALUE GetTINPoints (const RVC::OBJITEM& SrcObjItem, const TIN_PARAMETERS& TINParameters, POLYLINE& polyline);
		static ERRVALUE GetShapePoints (const RVC::OBJITEM& SrcObjItem, const SHAPE_PARAMETERS& ShapeParameters, POLYLINE& polyline);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}  // end namespace OP
}  // end namespace RVC


#endif  // INC_RVC_OPSURFACEFITTING_H
