/**
 * \file rvc/textobj.h 
 * \brief interface for the RVC::TEXTOBJ class.
 *
 * \if NODOC
 * $Id: textobj.h_v 1.29 2011/12/23 14:13:44 mju Exp $
 *
 * $Log: textobj.h_v $
 * Revision 1.29  2011/12/23 14:13:44  mju
 * Remove redundant hdrs.
 *
 * Revision 1.28  2006/10/13 19:24:31  mju
 * Use getobjfilter hdr.
 *
 * Revision 1.26  2006/06/27 15:05:09  scowan
 * Make method now fills in objitem with created object info.
 *
 * Revision 1.23  2005/12/09 17:30:00  scowan
 * Updated to use credentials.
 *
 * Revision 1.15  2005/02/08 21:10:51  scowan
 * Added methods for text obj type sets.
 *
 * Revision 1.8  2003/05/27 21:13:05  scowan
 * Added selection filter.
 *
 * Revision 1.1  2000/11/22 23:00:04  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_TEXTOBJ_H
#define INC_RVC_TEXTOBJ_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef INC_RVC_GETOBJFILTER_H
	#include <rvc/getobjfilter.h>
#endif


namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
struct TEXTINFO;
class TEXTHANDLE;
#endif

//! Manages RVC::OBJTYPE_Text and derived RVC objects.  
//!	
//!	RVC Object Types that this class supports: 
//!		- RVC::OBJTYPE_Script_Query
//!		- RVC::OBJTYPE_Script_SML
//!		- RVC::OBJTYPE_Display_Sim3D
//!		- RVC::OBJTYPE_Display_Group
//!		- RVC::OBJTYPE_Display_Layout
//!		- RVC::OBJTYPE_Disp2DWindow
//!		- RVC::OBJTYPE_DispParm
//!		- RVC::OBJTYPE_Text
//!		- RVC::OBJTYPE_DatabaseParm
//!		- RVC::OBJTYPE_Theme
//!		- RVC::OBJTYPE_Metadata
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Text.
//!
//!	Valid parents for this class:
//!		- All objects are valid parents
class TEXTOBJ : public RVC::OBJECT {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class GETOBJFILTER_HASTITLE;
		class GETOBJFILTER_ISSELECTABLE;
		friend class GETOBJFILTER_ISSELECTABLE;
		class MAKEPARMS;
		friend class MAKEPARMS;
		#endif
		
		//! Default constructor
		TEXTOBJ (
			);

		//! Copy constructor
		TEXTOBJ (
			const RVC::TEXTOBJ& obj
			);

		//! Destructor
		virtual ~TEXTOBJ (
			);

		//! Assignment
		RVC::TEXTOBJ& operator= (
			const RVC::TEXTOBJ& rhs
			);

		//! Append a string to the text object
		//! Appends new line value based on the OS
		//! @return: Number of characters written
		INT32 Append (
			const MIUNICODE *line
			);

		//! Write a MISTRING to the text object
		//! Appends new line value based on the OS
		//! @return: Number of characters written
		INT32 Append (
			const MISTRING& string
			);

		//! Get correct OBJTYPESET for all text OBJTYPE's
		//! @return OBJTYPESET for all text OBJTYPE's
		static RVC::OBJTYPESET GetObjTypeSet (
			);
			
		//! Determine if the object type is a text object			
		static bool IsText (
			RVC::OBJTYPE ObjType
			);

		//! Create a new rvc text object that is a link to an external text file
		ERRVALUE Link (
			const RVC::OBJECT& parent,									//!< Parent rvc object to create text object under
			const RVC::DESCRIPTOR& Descriptor,						//!< Name of new rvc text object
			const FILEPATH& LinkFile,									//!< Path to external text file to link to, will attempt relative path
#ifdef RVC_CREDENTIALS_MANDATORY
			CHAR_ENCODING encoding, 	//!< Encoding of text file
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			CHAR_ENCODING encoding = CHAR_ENCODING_ISO_2022,	//!< Encoding of text file
			const CREDENTIALS* Credentials = 0 						//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Create a new rvc text object
		ERRVALUE Make (
			const RVC::OBJECT& parent,				//!< Parent rvc object to create text object under
			const RVC::DESCRIPTOR& Descriptor,	//!< Name of new rvc text object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS 	//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new rvc text object
		ERRVALUE Make (
			const RVC::OBJECT& parent,				//!< Parent rvc object to create text object under
			const MAKEPARMS& MakeParms,
			const RVC::DESCRIPTOR& Descriptor,	//!< Name of new rvc text object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS 	//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new rvc text object
		ERRVALUE Make (
	    	RVC::OBJITEM& ObjItem,					//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,					//!< Source of rvc text object creator, default is GetAppName()
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,					//!< Source of rvc text object creator, default is GetAppName()
			const CREDENTIALS* Credentials = 0 	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Create a new rvc text object
		ERRVALUE Make (
	    	RVC::OBJITEM& ObjItem,					//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS& MakeParms,
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,					//!< Source of rvc text object creator, default is GetAppName()
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,					//!< Source of rvc text object creator, default is GetAppName()
			const CREDENTIALS* Credentials = 0 	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read text into the string, reads the whole object
		ERRVALUE Read (
			MISTRING& string						//!< String to read into
			);

		//! Read a line of text into the string
		//! This method cannot be used if current instance was opened with Make().
		//! This method does not remove end of line codes
		//!
		//! @return: EEndOfFile if read past the end or <0 of error or ==0 if no error
		ERRVALUE ReadLine (
			MISTRING& string						//!< String to read into
			);

		//! Reset the ReadLine() position to the beginning of the object			
		ERRVALUE Reset (
			);

		//! Write a string to the text object, replacing the string in the existing object
		ERRVALUE Write (
			const MISTRING& string
			);

	protected:

		void GetMetadataFormat (UINT8& format) const;
		void GetExtraValue (UINT32 offset, UINT16& value) const;
		void GetExtraValue (UINT32 offset, UINT32& value) const;
		ERRVALUE Link (const RVC::OBJECT& parent, OBJMAKEPARMS& ObjMakeParms, const FILEPATH& LinkFile, CHAR_ENCODING encoding = CHAR_ENCODING_ISO_2022);
		ERRVALUE Make (const OBJECT& parent, OBJMAKEPARMS& ObjMakeParms);
		ERRVALUE OpenObject (OPENMODE OpenFlags, TEXTINFO& newinfo, int HeaderSize, int HeaderType, MDLGPARENT parent, const CREDENTIALS* Credentials);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		
		class EXTRADATA {
			public:
				EXTRADATA () {}
				EXTRADATA (const EXTRADATA& rhs);
				EXTRADATA& operator=(const EXTRADATA& rhs);
				
				void Clear();
				
				void GetExtraValue (UINT32 offset, UINT8& value) const;
				void GetExtraValue (UINT32 offset, UINT16& value) const;
				void GetExtraValue (UINT32 offset, UINT32& value) const;
				
				bool IsEmpty () const { return (m_ExtraData.IsEmpty()); }
			
				void SetExtraValue (UINT32 offset, UINT8 value);
				void SetExtraValue (UINT32 offset, UINT16 value);
				void SetExtraValue (UINT32 offset, UINT32 value);
				
			private:
				SIMPLE_ARRAY<UINT8> m_ExtraData;
			};

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsMultiObject () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);
		virtual bool v_ValidateObjectType (const OBJTYPE otype) const;

		void SetupExtraHeaderData (const CREDENTIALS* Credentials) const;
		
		TEXTHANDLE* m_TextHandle;
		mutable EXTRADATA m_ExtraHeaderData;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Text object selection filter to determine if the text object is selectable under the parent object
class TEXTOBJ::GETOBJFILTER_ISSELECTABLE  : public GETOBJFILTER, private RVC::OBJECT::FILTER {
	public:
		GETOBJFILTER_ISSELECTABLE() {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool Filter (const RVC::OBJECT& Object);
		virtual bool v_HasSelectableSubObjects (const RVC::OBJITEM& objitem);
		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Text object selection filter to determine if the text object is selectable and has as its first line 
//! a title string matching the string given.
class TEXTOBJ::GETOBJFILTER_HASTITLE : public TEXTOBJ::GETOBJFILTER_ISSELECTABLE {
	public:
		explicit GETOBJFILTER_HASTITLE (
			const MISTRING& Title
			);
			
		explicit GETOBJFILTER_HASTITLE (
			const char* Title,
			CHAR_ENCODING encoding = CHAR_ENCODING_ASCII
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_Title;
		
		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class TEXTOBJ::MAKEPARMS {
	public:
		MAKEPARMS (
			RVC::OBJTYPE ObjType = RVC::OBJTYPE_Text
			);
	
		MAKEPARMS (
			const MAKEPARMS& rhs
			);
	
		MAKEPARMS (
			TEXTOBJ& TemplateObj
			);
	
		MAKEPARMS& operator= (
			const MAKEPARMS& rhs
			);
			
		RVC::OBJTYPE GetObjType (
			) const { return (m_ObjType); }
			
		void SetObjType (
			RVC::OBJTYPE ObjType
			) { m_ObjType = ObjType; }
			
	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		void GetExtraValue (UINT32 offset, UINT8& value) const;
		void GetExtraValue (UINT32 offset, UINT16& value) const;
		void GetExtraValue (UINT32 offset, UINT32& value) const;
		void SetExtraValue (UINT32 offset, UINT8 value);
		void SetExtraValue (UINT32 offset, UINT16 value);
		void SetExtraValue (UINT32 offset, UINT32 value);
		#endif // GENERATING_DOXYGEN_OUTPUT
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class TEXTOBJ;
		
		OBJTYPE m_ObjType;
		TEXTOBJ::EXTRADATA m_ExtraHeaderData;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif 	// INC_RVC_TEXT_H
