/**
 * \file soa/serverlist.h
 * \brief SOA SERVERLIST class definition
 *
 * \if NODOC
 * $Id: serverlist.h_v 1.9 2008/08/12 17:57:06 bcolombe Exp $
 *
 * $Log: serverlist.h_v $
 * Revision 1.9  2008/08/12 17:57:06  bcolombe
 * Added baseurl to item
 *
 * Revision 1.8  2008/02/08 21:28:00  bcolombe
 * Added TransferList by ITEMLIST
 *
 * Revision 1.7  2007/08/21 20:45:08  honerkamp
 * remove extra qualification
 *
 * Revision 1.6  2007/08/21 20:33:25  honerkamp
 * Add search option to server selection
 *
 * Revision 1.5  2007/04/09 16:05:17  mju
 * Remove downloadCatalog as done in dlgDownloadServerList.
 *
 * Revision 1.4  2007/02/23 22:01:20  mju
 * Add getService.
 *
 * Revision 1.3  2007/02/21 22:06:43  mju
 * Allow abstract for item.
 *
 * Revision 1.2  2007/02/12 21:13:07  mju
 * Add downloadCatalog.
 *
 * Revision 1.1  2007/02/09 22:11:16  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_SERVERLIST_H
#define	INC_SOA_SERVERLIST_H

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_STRUTF8_H
	#include <mi32/strutf8.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
	#include <mi32/filepath.h>
#endif

#include <vector>

class HTTPMESSAGE;
class XMLDOC;
class XMLNODE;

namespace SOA {

//=====================================================================================================================

//! Class to contain list of "servers" with title and URL for each.
//! Supports hierarchical "folders", local file load/save as well as loading from network reply.
class SERVERLIST {
	public:

		class ITEM;
		typedef MILIST<SERVERLIST::ITEM> ITEMLIST;

		//! Individual item in list.
		//! Contains title and URL for server and/or list of items if is itself a list.
		class ITEM {
			public:

				//! Default constructor
				ITEM (
					bool IsList = true
					);

				//! Constructor that creates an item with additional tags set
				ITEM (
					bool IsList,
					std::vector<MISTRING> additionaltags
					);

				//! Construct item to use as "list".
				ITEM (
					const MISTRING& title
					);

				//! Construct item referencing URL.
				ITEM (
					const MISTRING& title,
					const MISTRING& url,
					bool IsList = false
					);

				~ITEM ();

				//! Add child item to back of list.
				ITEM& AddChildToBack (
					const ITEM& item
					);

				//! Add child item to front of list.
				ITEM& AddChildToFront (
					const ITEM& item
					);

				//! Get abstract for this item.
				const MISTRING& GetAbstract (
					) const { return (m_Abstract); }

				//! Get the data for additional tags parsed out for this item
				const std::vector<MISTRING> GetAdditionalTagData (
						) const { return (m_AdditionalTagData); }

				//! Get the data for additional tags parsed out for this item
				const std::vector<MISTRING> GetAdditionalTagNames (
						) const { return (m_AdditionalTagNames); }

				//! Get list of items under this item.
				//! Should only be used if IsList() returns 'true'.
				const ITEMLIST& GetItemList (
					) const { return (m_ItemList); }

				//! Get pointer to parent item.
				//! @return NULL for topmost (root) item.
				const ITEM* GetParent (
					) const { return (m_pParent); }

				//! Get pointer to parent item.
				//! @return NULL for topmost (root) item.
				ITEM* GetParent (
					) { return (m_pParent); }

				//! Get item "service".
				//! @return Service string (eg, WMS, WFS, WCS, etc).  Uses STRUTF8 for convenience in comparison.
				const STRUTF8& GetService (
					) const { return (m_service); }

				//! Get item title.
				const MISTRING& GetTitle (
					) const { return (m_title); }

				//! Get item URL, will be empty if item is "list".
				const MISTRING& GetURL (
					) const { return (m_url); }

				//! Get item Base URL
				const MISTRING& GetBaseURL (
					) const { return (m_baseurl); }

				//! Insert child before specified iterator
				ITEM& InsertChild (
					ITEMLIST::CONST_ITERATOR itInsertBefore,
					const ITEM& item
					);

				//! Determine if item is a "list".
				bool IsList (
					) const { return (m_IsList); }

				//! Parse from XML node.
				//! Does not clear existing values, normally used on newly constructed item.
				ERRVALUE Parse (
					const XMLNODE* xmlnode
					);

				//! Remove child referenced by specified iterator.
				void RemoveChild (
					ITEMLIST::CONST_ITERATOR itChild
					);

				//! Set abstract.
				void SetAbstract (
					const MISTRING& Abstract
					) { m_Abstract = Abstract; }

				//! Set a stringlist of addition tags to parse out of the server list.
				void SetAdditionalTagNames (
					const std::vector<MISTRING>& additionaltags
					) { m_AdditionalTagNames = additionaltags;
						 m_AdditionalTagData.resize(additionaltags.size());}

				void SetAdditionalTagData (
					const std::vector<MISTRING>& additionaltags
					) { m_AdditionalTagData = additionaltags;}

				//! Set URL.
				void SetURL (
					const MISTRING& url
					) { m_url = url; }

				//! Set Base URL.
				void SetBaseURL (
					const MISTRING& baseurl
					) { m_baseurl = baseurl; }

					//! Set URL.
				void SetTitle (
					const MISTRING& title
					) { m_title = title; }

				//! Transfer list from specified item to this item.
				void TransferList (
					ITEM& listitem
					);

				//! Transfer list from specified item to this item.
				void TransferList (
					const ITEMLIST& list
					) {m_ItemList = list;}

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MISTRING m_title;
				MISTRING m_url;
				MISTRING m_baseurl;
				STRUTF8 m_service;
				MISTRING m_Abstract;
				std::vector<MISTRING> m_AdditionalTagNames;
				std::vector<MISTRING> m_AdditionalTagData;
				ITEMLIST m_ItemList;
				ITEM *m_pParent;
				bool m_IsList;

				ERRVALUE GetXML (XMLNODE *xmlnode) const;

				friend class SOA::SERVERLIST;
				#endif
			};

		//! Constructor.
		SERVERLIST ();

		//! Destructor.
		~SERVERLIST ();

		//! Get item list.
		const ITEMLIST& GetItemList (
			) const { return (m_TopItem.GetItemList()); }

		//! Get top (root) item.
		const ITEM& GetTopItem (
			) const { return (m_TopItem); }

		//! Get top (root) item.
		ITEM& GetTopItem (
			) { return (m_TopItem); }

		//! Load server list from file with optional additional tags to parse.
		ERRVALUE Load (
			const FILEPATH& filepath,
			std::vector<MISTRING> *additionaltags=0 
			);

		//! Load server list from message reply with optional additional tags to parse.
		ERRVALUE Load (
			const HTTPMESSAGE& msg,
			std::vector<MISTRING> *additionaltags=0
			);

		//! Save server list to file.
		ERRVALUE Save (
			const FILEPATH& filepath
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ITEM m_TopItem;

		ERRVALUE ParseXML (const XMLDOC& doc);
		#endif
	};

//=====================================================================================================================

}	// End namespace SOA

#endif	// INC_SOA_SERVERLIST_H
